@@ -57,8 +57,6 @@ struct _GstDVDemux {
   GstAdapter    *adapter;
   gint           frame_len;
 
-  /* PAL or NTSC flag */
-  gboolean       PAL;
   /* video params */
   gint           framerate_numerator;
   gint           framerate_denominator;
@@ -80,6 +78,7 @@ struct _GstDVDemux {
   gboolean       running;
   gboolean       need_segment;
   gboolean       new_media;
+  int            frames_since_new_media;
   
   gint           found_header; /* ATOMIC */
   GstEvent      *seek_event;
