@@ -28,16 +28,11 @@
 
 #include "gstgdkpixbuf.h"
 #include "gstgdkpixbufsink.h"
+#include "pixbufscale.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_gdk_pixbuf_debug);
 #define GST_CAT_DEFAULT gst_gdk_pixbuf_debug
 
-static const GstElementDetails plugin_details =
-GST_ELEMENT_DETAILS ("GdkPixbuf image decoder",
-    "Codec/Decoder/Image",
-    "Decodes images in a video stream using GdkPixbuf",
-    "David A. Schleef <ds@schleef.org>, Renato Filho <renato.filho@indt.org.br>");
-
 enum
 {
   ARG_0,
@@ -75,13 +70,6 @@ static GstStaticPadTemplate gst_gdk_pixbuf_src_template =
     GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_RGBA)
     );
 
-gboolean pixbufscale_init (GstPlugin * plugin);
-
-static void gst_gdk_pixbuf_base_init (gpointer g_class);
-static void gst_gdk_pixbuf_class_init (GstGdkPixbufClass * klass);
-static void gst_gdk_pixbuf_init (GstGdkPixbuf * filter,
-    GstGdkPixbufClass * klass);
-
 static void gst_gdk_pixbuf_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_gdk_pixbuf_get_property (GObject * object, guint prop_id,
@@ -130,6 +118,7 @@ gst_gdk_pixbuf_get_capslist (void)
   GSList *slist0;
   GstCaps *capslist = NULL;
   GstCaps *return_caps = NULL;
+  GstCaps *tmpl_caps;
 
   capslist = gst_caps_new_empty ();
   slist0 = gdk_pixbuf_get_formats ();
@@ -149,9 +138,10 @@ gst_gdk_pixbuf_get_capslist (void)
   }
   g_slist_free (slist0);
 
-  return_caps = gst_caps_intersect (capslist,
-      gst_static_caps_get (&gst_gdk_pixbuf_sink_template.static_caps));
+  tmpl_caps = gst_static_caps_get (&gst_gdk_pixbuf_sink_template.static_caps);
+  return_caps = gst_caps_intersect (capslist, tmpl_caps);
 
+  gst_caps_unref (tmpl_caps);
   gst_caps_unref (capslist);
   return return_caps;
 }
@@ -167,11 +157,14 @@ gst_gdk_pixbuf_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_gdk_pixbuf_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_gdk_pixbuf_sink_template));
-  gst_element_class_set_details (element_class, &plugin_details);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_gdk_pixbuf_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_gdk_pixbuf_sink_template);
+  gst_element_class_set_details_simple (element_class,
+      "GdkPixbuf image decoder", "Codec/Decoder/Image",
+      "Decodes images in a video stream using GdkPixbuf",
+      "David A. Schleef <ds@schleef.org>, Renato Filho <renato.filho@indt.org.br>");
 }
 
 /* initialize the plugin's class */
@@ -189,7 +182,8 @@ gst_gdk_pixbuf_class_init (GstGdkPixbufClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_SILENT,
       g_param_spec_boolean ("silent", "Silent",
-          "Produce verbose output ? (deprecated)", FALSE, G_PARAM_READWRITE));
+          "Produce verbose output ? (deprecated)", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_gdk_pixbuf_change_state);
@@ -420,10 +414,6 @@ static void
 gst_gdk_pixbuf_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstGdkPixbuf *filter;
-
-  filter = GST_GDK_PIXBUF (object);
-
   switch (prop_id) {
     case ARG_SILENT:
       /* filter->silent = g_value_get_boolean (value); */
@@ -438,10 +428,6 @@ static void
 gst_gdk_pixbuf_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GstGdkPixbuf *filter;
-
-  filter = GST_GDK_PIXBUF (object);
-
   switch (prop_id) {
     case ARG_SILENT:
       /* g_value_set_boolean (value, filter->silent); */
@@ -542,7 +528,7 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (gst_gdk_pixbuf_debug, "gdkpixbuf", 0,
       "gdk pixbuf loader");
 
-  if (!gst_element_register (plugin, "gdkpixbufdec", GST_RANK_MARGINAL,
+  if (!gst_element_register (plugin, "gdkpixbufdec", GST_RANK_SECONDARY,
           GST_TYPE_GDK_PIXBUF))
     return FALSE;
 
