@@ -24,25 +24,28 @@
 /**
  * SECTION:element-wavpackparse
  *
- * <refsect2>
  * WavpackParse takes raw, unframed Wavpack streams and splits them into
  * single Wavpack chunks with information like bit depth and the position
  * in the stream.
  * <ulink url="http://www.wavpack.com/">Wavpack</ulink> is an open-source
  * audio codec that features both lossless and lossy encoding.
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=test.wv ! wavpackparse ! wavpackdec ! fakesink
- * </programlisting>
- * This pipeline decodes the Wavpack file test.wv into raw audio buffers.
- * </para>
+ * ]| This pipeline decodes the Wavpack file test.wv into raw audio buffers.
  * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
 
@@ -57,6 +60,18 @@
 GST_DEBUG_CATEGORY_STATIC (gst_wavpack_parse_debug);
 #define GST_CAT_DEFAULT gst_wavpack_parse_debug
 
+static inline GstWavpackParseIndexEntry *
+gst_wavpack_parse_index_entry_new (void)
+{
+  return g_slice_new (GstWavpackParseIndexEntry);
+}
+
+static inline void
+gst_wavpack_parse_index_entry_free (GstWavpackParseIndexEntry * entry)
+{
+  g_slice_free (GstWavpackParseIndexEntry, entry);
+}
+
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -81,14 +96,18 @@ static GstStaticPadTemplate wvc_src_factory = GST_STATIC_PAD_TEMPLATE ("wvcsrc",
     );
 
 static gboolean gst_wavpack_parse_sink_activate (GstPad * sinkpad);
+
 static gboolean
 gst_wavpack_parse_sink_activate_pull (GstPad * sinkpad, gboolean active);
 
 static void gst_wavpack_parse_loop (GstElement * element);
+
 static GstStateChangeReturn gst_wavpack_parse_change_state (GstElement *
     element, GstStateChange transition);
 static void gst_wavpack_parse_reset (GstWavpackParse * parse);
+
 static gint64 gst_wavpack_parse_get_upstream_length (GstWavpackParse * wvparse);
+
 static GstBuffer *gst_wavpack_parse_pull_buffer (GstWavpackParse * wvparse,
     gint64 offset, guint size, GstFlowReturn * flow);
 static GstFlowReturn gst_wavpack_parse_chain (GstPad * pad, GstBuffer * buf);
@@ -99,21 +118,18 @@ GST_BOILERPLATE (GstWavpackParse, gst_wavpack_parse, GstElement,
 static void
 gst_wavpack_parse_base_init (gpointer klass)
 {
-  static const GstElementDetails plugin_details =
-      GST_ELEMENT_DETAILS ("Wavpack parser",
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &wvc_src_factory);
+  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+
+  gst_element_class_set_details_simple (element_class, "Wavpack parser",
       "Codec/Demuxer/Audio",
       "Parses Wavpack files",
       "Arwed v. Merkatz <v.merkatz@gmx.net>, "
       "Sebastian Dröge <slomo@circular-chaos.org>");
-  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&wvc_src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory));
-  gst_element_class_set_details (element_class, &plugin_details);
 }
 
 static void
@@ -128,12 +144,13 @@ static void
 gst_wavpack_parse_class_init (GstWavpackParseClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_wavpack_parse_finalize);
+  gobject_class->finalize = gst_wavpack_parse_finalize;
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_wavpack_parse_change_state);
 }
@@ -151,6 +168,7 @@ gst_wavpack_parse_index_get_entry_from_sample (GstWavpackParse * wvparse,
     gint64 sample_offset)
 {
   gint i;
+
   GSList *node;
 
   if (wvparse->entries == NULL)
@@ -199,7 +217,7 @@ gst_wavpack_parse_index_append_entry (GstWavpackParse * wvparse,
       GST_TIME_ARGS (gst_util_uint64_scale_int (sample_offset,
               GST_SECOND, wvparse->samplerate)), byte_offset);
 
-  entry = g_new0 (GstWavpackParseIndexEntry, 1);
+  entry = gst_wavpack_parse_index_entry_new ();
   entry->byte_offset = byte_offset;
   entry->sample_offset = sample_offset;
   entry->sample_offset_end = sample_offset + num_samples;
@@ -209,7 +227,7 @@ gst_wavpack_parse_index_append_entry (GstWavpackParse * wvparse,
 static void
 gst_wavpack_parse_reset (GstWavpackParse * parse)
 {
-  parse->total_samples = -1;
+  parse->total_samples = G_GINT64_CONSTANT (-1);
   parse->samplerate = 0;
   parse->channels = 0;
 
@@ -222,7 +240,8 @@ gst_wavpack_parse_reset (GstWavpackParse * parse)
   parse->upstream_length = -1;
 
   if (parse->entries) {
-    g_slist_foreach (parse->entries, (GFunc) g_free, NULL);
+    g_slist_foreach (parse->entries, (GFunc) gst_wavpack_parse_index_entry_free,
+        NULL);
     g_slist_free (parse->entries);
     parse->entries = NULL;
   }
@@ -270,21 +289,23 @@ static gboolean
 gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
 {
   GstWavpackParse *parse = GST_WAVPACK_PARSE (gst_pad_get_parent (pad));
+
   GstFormat format;
+
   gboolean ret = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:{
-      gint64 cur, len;
+      gint64 cur;
+
       guint rate;
 
       GST_OBJECT_LOCK (parse);
       cur = parse->segment.last_stop;
-      len = parse->total_samples;
       rate = parse->samplerate;
       GST_OBJECT_UNLOCK (parse);
 
-      if (len < 0 || rate == 0) {
+      if (rate == 0) {
         GST_DEBUG_OBJECT (parse, "haven't read header yet");
         break;
       }
@@ -311,16 +332,15 @@ gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
     }
     case GST_QUERY_DURATION:{
       gint64 len;
+
       guint rate;
 
       GST_OBJECT_LOCK (parse);
       rate = parse->samplerate;
-      /* FIXME: return 0 if we work in push based mode to let totem
-       * recognize that we can't seek */
-      len = (parse->adapter) ? 0 : parse->total_samples;
+      len = parse->total_samples;
       GST_OBJECT_UNLOCK (parse);
 
-      if (len < 0 || rate == 0) {
+      if (rate == 0) {
         GST_DEBUG_OBJECT (parse, "haven't read header yet");
         break;
       }
@@ -329,7 +349,8 @@ gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
 
       switch (format) {
         case GST_FORMAT_TIME:
-          len = gst_util_uint64_scale_int (len, GST_SECOND, rate);
+          if (len != G_GINT64_CONSTANT (-1))
+            len = gst_util_uint64_scale_int (len, GST_SECOND, rate);
           gst_query_set_duration (query, GST_FORMAT_TIME, len);
           ret = TRUE;
           break;
@@ -349,6 +370,7 @@ gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
       gst_query_parse_seeking (query, &format, NULL, NULL, NULL);
       if (format == GST_FORMAT_TIME || format == GST_FORMAT_DEFAULT) {
         gboolean seekable;
+
         gint64 duration = -1;
 
         /* only fails if we didn't read the headers yet and can't say
@@ -389,7 +411,9 @@ gst_wavpack_parse_scan_to_find_sample (GstWavpackParse * parse,
     gint64 sample, gint64 * byte_offset, gint64 * start_sample)
 {
   GstWavpackParseIndexEntry *entry;
+
   GstFlowReturn ret;
+
   gint64 off = 0;
 
   /* first, check if we have to scan at all */
@@ -417,6 +441,7 @@ gst_wavpack_parse_scan_to_find_sample (GstWavpackParse * parse,
   /* now scan forward until we find the chunk we're looking for or hit EOS */
   do {
     WavpackHeader header;
+
     GstBuffer *buf;
 
     buf = gst_wavpack_parse_pull_buffer (parse, off, sizeof (WavpackHeader),
@@ -454,10 +479,15 @@ static gboolean
 gst_wavpack_parse_send_newsegment (GstWavpackParse * wvparse, gboolean update)
 {
   GstSegment *s = &wvparse->segment;
+
   gboolean ret;
+
   gint64 stop_time = -1;
+
   gint64 start_time = 0;
+
   gint64 cur_pos_time;
+
   gint64 diff;
 
   /* segment is in DEFAULT format, but we want to send a TIME newsegment */
@@ -494,18 +524,31 @@ gst_wavpack_parse_handle_seek_event (GstWavpackParse * wvparse,
     GstEvent * event)
 {
   GstSeekFlags seek_flags;
+
   GstSeekType start_type;
+
   GstSeekType stop_type;
+
   GstSegment segment;
+
   GstFormat format;
+
   gboolean only_update;
+
   gboolean flush, ret;
+
   gdouble speed;
+
   gint64 stop;
+
   gint64 start;                 /* sample we want to seek to                  */
+
   gint64 byte_offset;           /* byte offset the chunk we seek to starts at */
+
   gint64 chunk_start;           /* first sample in chunk we seek to           */
+
   guint rate;
+
   gint64 last_stop;
 
   if (wvparse->adapter) {
@@ -631,6 +674,7 @@ static gboolean
 gst_wavpack_parse_sink_event (GstPad * pad, GstEvent * event)
 {
   GstWavpackParse *parse;
+
   gboolean ret = TRUE;
 
   parse = GST_WAVPACK_PARSE (gst_pad_get_parent (pad));
@@ -689,6 +733,7 @@ static gboolean
 gst_wavpack_parse_src_event (GstPad * pad, GstEvent * event)
 {
   GstWavpackParse *parse;
+
   gboolean ret;
 
   parse = GST_WAVPACK_PARSE (gst_pad_get_parent (pad));
@@ -710,6 +755,7 @@ static void
 gst_wavpack_parse_init (GstWavpackParse * parse, GstWavpackParseClass * gclass)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (parse);
+
   GstPadTemplate *tmpl;
 
   tmpl = gst_element_class_get_pad_template (klass, "sink");
@@ -750,11 +796,12 @@ gst_wavpack_parse_pull_buffer (GstWavpackParse * wvparse, gint64 offset,
     guint size, GstFlowReturn * flow)
 {
   GstFlowReturn flow_ret;
+
   GstBuffer *buf = NULL;
 
-  if (offset + size >= wvparse->upstream_length) {
+  if (offset + size > wvparse->upstream_length) {
     wvparse->upstream_length = gst_wavpack_parse_get_upstream_length (wvparse);
-    if (offset + size >= wvparse->upstream_length) {
+    if (offset + size > wvparse->upstream_length) {
       GST_DEBUG_OBJECT (wvparse, "EOS: %" G_GINT64_FORMAT " + %u > %"
           G_GINT64_FORMAT, offset, size, wvparse->upstream_length);
       flow_ret = GST_FLOW_UNEXPECTED;
@@ -790,7 +837,9 @@ gst_wavpack_parse_create_src_pad (GstWavpackParse * wvparse, GstBuffer * buf,
     WavpackHeader * header)
 {
   GstWavpackMetadata meta;
+
   GstCaps *caps = NULL;
+
   guchar *bufptr;
 
   g_assert (wvparse->srcpad == NULL);
@@ -810,9 +859,13 @@ gst_wavpack_parse_create_src_pad (GstWavpackParse * wvparse, GstBuffer * buf,
       case ID_WV_BITSTREAM:
       case ID_WVX_BITSTREAM:{
         WavpackStreamReader *stream_reader = gst_wavpack_stream_reader_new ();
+
         WavpackContext *wpc;
+
         gchar error_msg[80];
+
         read_id rid;
+
         gint channel_mask;
 
         rid.buffer = GST_BUFFER_DATA (buf);
@@ -827,9 +880,9 @@ gst_wavpack_parse_create_src_pad (GstWavpackParse * wvparse, GstBuffer * buf,
 
         wvparse->samplerate = WavpackGetSampleRate (wpc);
         wvparse->channels = WavpackGetNumChannels (wpc);
-        wvparse->total_samples = header->total_samples;
-        if (wvparse->total_samples == (int32_t) - 1)
-          wvparse->total_samples = 0;
+        wvparse->total_samples =
+            (header->total_samples ==
+            0xffffffff) ? G_GINT64_CONSTANT (-1) : header->total_samples;
 
         caps = gst_caps_new_simple ("audio/x-wavpack",
             "width", G_TYPE_INT, WavpackGetBitsPerSample (wpc),
@@ -900,6 +953,7 @@ static GstFlowReturn
 gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
     WavpackHeader * header)
 {
+  GstFlowReturn ret;
   wvparse->current_offset += header->ckSize + 8;
 
   wvparse->segment.last_stop = header->block_index;
@@ -957,13 +1011,18 @@ gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
   GST_LOG_OBJECT (wvparse, "Pushing buffer with time %" GST_TIME_FORMAT,
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
 
-  return gst_pad_push (wvparse->srcpad, buf);
+  ret = gst_pad_push (wvparse->srcpad, buf);
+
+  wvparse->segment.last_stop = wvparse->next_block_index;
+
+  return ret;
 }
 
 static guint8 *
 gst_wavpack_parse_find_marker (guint8 * buf, guint size)
 {
   int i;
+
   guint8 *ret = NULL;
 
   if (G_UNLIKELY (size < 4))
@@ -982,11 +1041,13 @@ static GstFlowReturn
 gst_wavpack_parse_resync_loop (GstWavpackParse * parse, WavpackHeader * header)
 {
   GstFlowReturn flow_ret = GST_FLOW_UNEXPECTED;
+
   GstBuffer *buf = NULL;
 
   /* loop until we have a frame header or reach the end of the stream */
   while (1) {
     guint8 *data, *marker;
+
     guint len, size;
 
     if (buf) {
@@ -1047,6 +1108,7 @@ static void
 gst_wavpack_parse_loop (GstElement * element)
 {
   GstWavpackParse *parse = GST_WAVPACK_PARSE (element);
+
   GstFlowReturn flow_ret;
   WavpackHeader header = { {0,}, 0, };
   GstBuffer *buf = NULL;
@@ -1089,30 +1151,29 @@ pause:
     GST_LOG_OBJECT (parse, "pausing task, reason %s", reason);
     gst_pad_pause_task (parse->sinkpad);
 
-    if (GST_FLOW_IS_FATAL (flow_ret) || flow_ret == GST_FLOW_NOT_LINKED) {
-      if (flow_ret == GST_FLOW_UNEXPECTED && parse->srcpad) {
-        if (parse->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-          GstClockTime stop;
+    if (flow_ret == GST_FLOW_UNEXPECTED && parse->srcpad) {
+      if (parse->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+        GstClockTime stop;
 
-          GST_LOG_OBJECT (parse, "Sending segment done");
+        GST_LOG_OBJECT (parse, "Sending segment done");
 
-          if ((stop = parse->segment.stop) == -1)
-            stop = parse->segment.duration;
+        if ((stop = parse->segment.stop) == -1)
+          stop = parse->segment.duration;
 
-          gst_element_post_message (GST_ELEMENT_CAST (parse),
-              gst_message_new_segment_done (GST_OBJECT_CAST (parse),
-                  parse->segment.format, stop));
-        } else {
-          GST_LOG_OBJECT (parse, "Sending EOS, at end of stream");
-          gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
-        }
+        gst_element_post_message (GST_ELEMENT_CAST (parse),
+            gst_message_new_segment_done (GST_OBJECT_CAST (parse),
+                parse->segment.format, stop));
       } else {
-        GST_ELEMENT_ERROR (parse, STREAM, FAILED,
-            (_("Internal data stream error.")),
-            ("stream stopped, reason %s", reason));
-        if (parse->srcpad)
-          gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
+        GST_LOG_OBJECT (parse, "Sending EOS, at end of stream");
+        gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
       }
+    } else if (flow_ret == GST_FLOW_NOT_LINKED
+        || flow_ret < GST_FLOW_UNEXPECTED) {
+      GST_ELEMENT_ERROR (parse, STREAM, FAILED,
+          (_("Internal data stream error.")), ("stream stopped, reason %s",
+              reason));
+      if (parse->srcpad)
+        gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
     }
     return;
   }
@@ -1122,6 +1183,7 @@ static gboolean
 gst_wavpack_parse_resync_adapter (GstAdapter * adapter)
 {
   const guint8 *buf, *marker;
+
   guint avail = gst_adapter_available (adapter);
 
   if (avail < 4)
@@ -1153,8 +1215,11 @@ static GstFlowReturn
 gst_wavpack_parse_chain (GstPad * pad, GstBuffer * buf)
 {
   GstWavpackParse *wvparse = GST_WAVPACK_PARSE (GST_PAD_PARENT (pad));
+
   GstFlowReturn ret = GST_FLOW_OK;
+
   WavpackHeader wph;
+
   const guint8 *tmp_buf;
 
   if (!wvparse->adapter) {
@@ -1214,6 +1279,7 @@ static GstStateChangeReturn
 gst_wavpack_parse_change_state (GstElement * element, GstStateChange transition)
 {
   GstWavpackParse *wvparse = GST_WAVPACK_PARSE (element);
+
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   switch (transition) {
