@@ -22,13 +22,14 @@
  * SECTION:element-goom
  * @see_also: synaesthesia
  *
+ * Goom is an audio visualisation element. It creates warping structures
+ * based on the incoming audio signal.
+ *
  * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v audiotestsrc ! goom ! ffmpegcolorspace ! xvimagesink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -40,17 +41,19 @@
 #include <gst/gst.h>
 #include "gstgoom.h"
 #include <gst/video/video.h>
-#include "goom_core.h"
+#include "goom.h"
 
-GST_DEBUG_CATEGORY_STATIC (goom_debug);
+#if HAVE_ORC
+#include <orc/orc.h>
+#endif
+
+GST_DEBUG_CATEGORY (goom_debug);
 #define GST_CAT_DEFAULT goom_debug
 
-/* elementfactory information */
-static const GstElementDetails gst_goom_details =
-GST_ELEMENT_DETAILS ("GOOM: what a GOOM!",
-    "Visualization",
-    "Takes frames of data and outputs video frames using the GOOM filter",
-    "Wim Taymans <wim@fluendo.com>");
+#define DEFAULT_WIDTH  320
+#define DEFAULT_HEIGHT 240
+#define DEFAULT_FPS_N  25
+#define DEFAULT_FPS_D  1
 
 /* signals and args */
 enum
@@ -95,6 +98,8 @@ static GstFlowReturn gst_goom_chain (GstPad * pad, GstBuffer * buffer);
 static gboolean gst_goom_src_event (GstPad * pad, GstEvent * event);
 static gboolean gst_goom_sink_event (GstPad * pad, GstEvent * event);
 
+static gboolean gst_goom_src_query (GstPad * pad, GstQuery * query);
+
 static gboolean gst_goom_sink_setcaps (GstPad * pad, GstCaps * caps);
 static gboolean gst_goom_src_setcaps (GstPad * pad, GstCaps * caps);
 
@@ -128,11 +133,13 @@ gst_goom_base_init (GstGoomClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &gst_goom_details);
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_set_details_simple (element_class, "GOOM: what a GOOM!",
+      "Visualization",
+      "Takes frames of data and outputs video frames using the GOOM filter",
+      "Wim Taymans <wim@fluendo.com>");
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template);
+  gst_element_class_add_static_pad_template (element_class, &src_template);
 }
 
 static void
@@ -148,9 +155,7 @@ gst_goom_class_init (GstGoomClass * klass)
 
   gobject_class->finalize = gst_goom_finalize;
 
-  gstelement_class->change_state = gst_goom_change_state;
-
-  GST_DEBUG_CATEGORY_INIT (goom_debug, "goom", 0, "goom visualisation element");
+  gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_goom_change_state);
 }
 
 static void
@@ -158,27 +163,34 @@ gst_goom_init (GstGoom * goom)
 {
   /* create the sink and src pads */
   goom->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
-  gst_pad_set_chain_function (goom->sinkpad, gst_goom_chain);
-  gst_pad_set_event_function (goom->sinkpad, gst_goom_sink_event);
-  gst_pad_set_setcaps_function (goom->sinkpad, gst_goom_sink_setcaps);
+  gst_pad_set_chain_function (goom->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_goom_chain));
+  gst_pad_set_event_function (goom->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_goom_sink_event));
+  gst_pad_set_setcaps_function (goom->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_goom_sink_setcaps));
   gst_element_add_pad (GST_ELEMENT (goom), goom->sinkpad);
 
   goom->srcpad = gst_pad_new_from_static_template (&src_template, "src");
-  gst_pad_set_setcaps_function (goom->srcpad, gst_goom_src_setcaps);
-  gst_pad_set_event_function (goom->srcpad, gst_goom_src_event);
+  gst_pad_set_setcaps_function (goom->srcpad,
+      GST_DEBUG_FUNCPTR (gst_goom_src_setcaps));
+  gst_pad_set_event_function (goom->srcpad,
+      GST_DEBUG_FUNCPTR (gst_goom_src_event));
+  gst_pad_set_query_function (goom->srcpad,
+      GST_DEBUG_FUNCPTR (gst_goom_src_query));
   gst_element_add_pad (GST_ELEMENT (goom), goom->srcpad);
 
   goom->adapter = gst_adapter_new ();
 
-  goom->width = 320;
-  goom->height = 200;
-  goom->fps_n = 25;             /* desired frame rate */
-  goom->fps_d = 1;              /* desired frame rate */
+  goom->width = DEFAULT_WIDTH;
+  goom->height = DEFAULT_HEIGHT;
+  goom->fps_n = DEFAULT_FPS_N;  /* desired frame rate */
+  goom->fps_d = DEFAULT_FPS_D;  /* desired frame rate */
   goom->channels = 0;
   goom->rate = 0;
   goom->duration = 0;
 
-  goom_init (&(goom->goomdata), goom->width, goom->height);
+  goom->plugin = goom_init (goom->width, goom->height);
 }
 
 static void
@@ -186,7 +198,8 @@ gst_goom_finalize (GObject * object)
 {
   GstGoom *goom = GST_GOOM (object);
 
-  goom_close (&(goom->goomdata));
+  goom_close (goom->plugin);
+  goom->plugin = NULL;
 
   g_object_unref (goom->adapter);
 
@@ -196,7 +209,6 @@ gst_goom_finalize (GObject * object)
 static void
 gst_goom_reset (GstGoom * goom)
 {
-  goom->next_ts = -1;
   gst_adapter_clear (goom->adapter);
   gst_segment_init (&goom->segment, GST_FORMAT_UNDEFINED);
 
@@ -241,13 +253,14 @@ gst_goom_src_setcaps (GstPad * pad, GstCaps * caps)
           &goom->fps_d))
     return FALSE;
 
-  goom_set_resolution (&(goom->goomdata), goom->width, goom->height);
+  goom_set_resolution (goom->plugin, goom->width, goom->height);
 
   /* size of the output buffer in bytes, depth is always 4 bytes */
   goom->outsize = goom->width * goom->height * 4;
   goom->duration =
       gst_util_uint64_scale_int (GST_SECOND, goom->fps_d, goom->fps_n);
   goom->spf = gst_util_uint64_scale_int (goom->rate, goom->fps_d, goom->fps_n);
+  goom->bpf = goom->spf * goom->bps;
 
   GST_DEBUG_OBJECT (goom, "dimension %dx%d, framerate %d/%d, spf %d",
       goom->width, goom->height, goom->fps_n, goom->fps_d, goom->spf);
@@ -258,7 +271,7 @@ gst_goom_src_setcaps (GstPad * pad, GstCaps * caps)
 static gboolean
 gst_goom_src_negotiate (GstGoom * goom)
 {
-  GstCaps *othercaps, *target, *intersect;
+  GstCaps *othercaps, *target;
   GstStructure *structure;
   const GstCaps *templ;
 
@@ -269,22 +282,22 @@ gst_goom_src_negotiate (GstGoom * goom)
   /* see what the peer can do */
   othercaps = gst_pad_peer_get_caps (goom->srcpad);
   if (othercaps) {
-    intersect = gst_caps_intersect (othercaps, templ);
+    target = gst_caps_intersect (othercaps, templ);
     gst_caps_unref (othercaps);
 
-    if (gst_caps_is_empty (intersect))
+    if (gst_caps_is_empty (target))
       goto no_format;
 
-    target = gst_caps_copy_nth (intersect, 0);
-    gst_caps_unref (intersect);
+    gst_caps_truncate (target);
   } else {
     target = gst_caps_ref ((GstCaps *) templ);
   }
 
   structure = gst_caps_get_structure (target, 0);
-  gst_structure_fixate_field_nearest_int (structure, "width", 320);
-  gst_structure_fixate_field_nearest_int (structure, "height", 240);
-  gst_structure_fixate_field_nearest_fraction (structure, "framerate", 30, 1);
+  gst_structure_fixate_field_nearest_int (structure, "width", DEFAULT_WIDTH);
+  gst_structure_fixate_field_nearest_int (structure, "height", DEFAULT_HEIGHT);
+  gst_structure_fixate_field_nearest_fraction (structure, "framerate",
+      DEFAULT_FPS_N, DEFAULT_FPS_D);
 
   gst_pad_set_caps (goom->srcpad, target);
   gst_caps_unref (target);
@@ -293,7 +306,7 @@ gst_goom_src_negotiate (GstGoom * goom)
 
 no_format:
   {
-    gst_caps_unref (intersect);
+    gst_caps_unref (target);
     return FALSE;
   }
 }
@@ -383,6 +396,66 @@ gst_goom_sink_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static gboolean
+gst_goom_src_query (GstPad * pad, GstQuery * query)
+{
+  gboolean res = FALSE;
+  GstGoom *goom;
+
+  goom = GST_GOOM (gst_pad_get_parent (pad));
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:
+    {
+      /* We need to send the query upstream and add the returned latency to our
+       * own */
+      GstClockTime min_latency, max_latency;
+      gboolean us_live;
+      GstClockTime our_latency;
+      guint max_samples;
+
+      if (goom->rate == 0)
+        break;
+
+      if ((res = gst_pad_peer_query (goom->sinkpad, query))) {
+        gst_query_parse_latency (query, &us_live, &min_latency, &max_latency);
+
+        GST_DEBUG_OBJECT (goom, "Peer latency: min %"
+            GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
+
+        /* the max samples we must buffer buffer */
+        max_samples = MAX (GOOM_SAMPLES, goom->spf);
+        our_latency =
+            gst_util_uint64_scale_int (max_samples, GST_SECOND, goom->rate);
+
+        GST_DEBUG_OBJECT (goom, "Our latency: %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (our_latency));
+
+        /* we add some latency but only if we need to buffer more than what
+         * upstream gives us */
+        min_latency += our_latency;
+        if (max_latency != -1)
+          max_latency += our_latency;
+
+        GST_DEBUG_OBJECT (goom, "Calculated total latency : min %"
+            GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
+
+        gst_query_set_latency (query, TRUE, min_latency, max_latency);
+      }
+      break;
+    }
+    default:
+      res = gst_pad_peer_query (goom->sinkpad, query);
+      break;
+  }
+
+  gst_object_unref (goom);
+
+  return res;
+}
+
 static GstFlowReturn
 get_buffer (GstGoom * goom, GstBuffer ** outbuf)
 {
@@ -400,13 +473,9 @@ get_buffer (GstGoom * goom, GstBuffer ** outbuf)
       gst_pad_alloc_buffer_and_set_caps (goom->srcpad,
       GST_BUFFER_OFFSET_NONE, goom->outsize,
       GST_PAD_CAPS (goom->srcpad), outbuf);
-
   if (ret != GST_FLOW_OK)
     return ret;
 
-  if (*outbuf == NULL)
-    return GST_FLOW_ERROR;
-
   return GST_FLOW_OK;
 }
 
@@ -416,9 +485,12 @@ gst_goom_chain (GstPad * pad, GstBuffer * buffer)
   GstGoom *goom;
   GstFlowReturn ret;
   GstBuffer *outbuf = NULL;
-  guint avail;
 
   goom = GST_GOOM (gst_pad_get_parent (pad));
+  if (goom->bps == 0) {
+    ret = GST_FLOW_NOT_NEGOTIATED;
+    goto beach;
+  }
 
   /* If we don't have an output format yet, preallocate a buffer to try and
    * set one */
@@ -433,37 +505,52 @@ gst_goom_chain (GstPad * pad, GstBuffer * buffer)
   /* don't try to combine samples from discont buffer */
   if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
     gst_adapter_clear (goom->adapter);
-    goom->next_ts = -1;
   }
 
-  /* Match timestamps from the incoming audio */
-  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE)
-    goom->next_ts = GST_BUFFER_TIMESTAMP (buffer);
-
   GST_DEBUG_OBJECT (goom,
       "Input buffer has %d samples, time=%" G_GUINT64_FORMAT,
       GST_BUFFER_SIZE (buffer) / goom->bps, GST_BUFFER_TIMESTAMP (buffer));
 
+  /* Collect samples until we have enough for an output frame */
   gst_adapter_push (goom->adapter, buffer);
 
   ret = GST_FLOW_OK;
 
-  /* Collect samples until we have enough for an output frame */
-  avail = gst_adapter_available (goom->adapter);
-  GST_DEBUG_OBJECT (goom, "avail now %u", avail);
-  while (avail > MAX (GOOM_SAMPLES, goom->spf) * goom->bps) {
+  while (TRUE) {
     const guint16 *data;
     gboolean need_skip;
     guchar *out_frame;
     gint i;
+    guint avail, to_flush;
+    guint64 dist, timestamp;
+
+    avail = gst_adapter_available (goom->adapter);
+    GST_DEBUG_OBJECT (goom, "avail now %u", avail);
+
+    /* we need GOOM_SAMPLES to get a meaningful result from goom. */
+    if (avail < (GOOM_SAMPLES * goom->bps))
+      break;
+
+    /* we also need enough samples to produce one frame at least */
+    if (avail < goom->bpf)
+      break;
 
     GST_DEBUG_OBJECT (goom, "processing buffer");
 
-    if (goom->next_ts != -1) {
+    /* get timestamp of the current adapter byte */
+    timestamp = gst_adapter_prev_timestamp (goom->adapter, &dist);
+    if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      /* convert bytes to time */
+      dist /= goom->bps;
+      timestamp += gst_util_uint64_scale_int (dist, GST_SECOND, goom->rate);
+    }
+
+    if (timestamp != -1) {
       gint64 qostime;
 
       qostime = gst_segment_to_running_time (&goom->segment, GST_FORMAT_TIME,
-          goom->next_ts);
+          timestamp);
+      qostime += goom->duration;
 
       GST_OBJECT_LOCK (goom);
       /* check for QoS, don't compute buffers that are known to be late */
@@ -478,6 +565,7 @@ gst_goom_chain (GstPad * pad, GstBuffer * buffer)
       }
     }
 
+    /* get next GOOM_SAMPLES, we have at least this amount of samples */
     data =
         (const guint16 *) gst_adapter_peek (goom->adapter,
         GOOM_SAMPLES * goom->bps);
@@ -503,34 +591,31 @@ gst_goom_chain (GstPad * pad, GstBuffer * buffer)
       }
     }
 
-    GST_BUFFER_TIMESTAMP (outbuf) = goom->next_ts;
+    GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
     GST_BUFFER_DURATION (outbuf) = goom->duration;
     GST_BUFFER_SIZE (outbuf) = goom->outsize;
 
-    out_frame = (guchar *) goom_update (&(goom->goomdata), goom->datain);
+    out_frame = (guchar *) goom_update (goom->plugin, goom->datain, 0, 0);
     memcpy (GST_BUFFER_DATA (outbuf), out_frame, goom->outsize);
 
     GST_DEBUG ("Pushing frame with time=%" GST_TIME_FORMAT ", duration=%"
-        GST_TIME_FORMAT, GST_TIME_ARGS (goom->next_ts),
+        GST_TIME_FORMAT, GST_TIME_ARGS (timestamp),
         GST_TIME_ARGS (goom->duration));
 
     ret = gst_pad_push (goom->srcpad, outbuf);
     outbuf = NULL;
 
   skip:
-    /* interpollate next timestamp */
-    if (goom->next_ts != -1)
-      goom->next_ts += goom->duration;
+    /* Now flush the samples we needed for this frame, which might be more than
+     * the samples we used (GOOM_SAMPLES). */
+    to_flush = goom->bpf;
 
-    GST_DEBUG_OBJECT (goom, "finished frame, flushing %u samples from input",
-        goom->spf);
-    gst_adapter_flush (goom->adapter, MIN (avail, goom->spf * goom->bps));
+    GST_DEBUG_OBJECT (goom, "finished frame, flushing %u bytes from input",
+        to_flush);
+    gst_adapter_flush (goom->adapter, to_flush);
 
     if (ret != GST_FLOW_OK)
       break;
-
-    avail = gst_adapter_available (goom->adapter);
-    GST_DEBUG_OBJECT (goom, "avail now %u", avail);
   }
 
   if (outbuf != NULL)
@@ -548,7 +633,6 @@ gst_goom_change_state (GstElement * element, GstStateChange transition)
   GstGoom *goom = GST_GOOM (element);
   GstStateChangeReturn ret;
 
-
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
@@ -576,6 +660,12 @@ gst_goom_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  GST_DEBUG_CATEGORY_INIT (goom_debug, "goom", 0, "goom visualisation element");
+
+#if HAVE_ORC
+  orc_init ();
+#endif
+
   return gst_element_register (plugin, "goom", GST_RANK_NONE, GST_TYPE_GOOM);
 }
 
