@@ -1,5 +1,20 @@
-/*
- *  lines.c
+/* Goom Project
+ * Copyright (C) <2003> iOS-Software
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "lines.h"
@@ -55,7 +70,7 @@ genline (int id, float param, GMUnitPointer * l, int rx, int ry)
       for (i = 0; i < 512; i++) {
         l[i].x = ((float) i * rx) / 512.0f;
         l[i].y = param;
-        l[i].angle = M_PI / 2.0f;
+        l[i].angle = G_PI / 2.0f;
       }
       return;
     case GML_VLINE:
@@ -69,7 +84,7 @@ genline (int id, float param, GMUnitPointer * l, int rx, int ry)
       for (i = 0; i < 512; i++) {
         float cosa, sina;
 
-        l[i].angle = 2.0f * M_PI * (float) i / 512.0f;
+        l[i].angle = 2.0f * G_PI * (float) i / 512.0f;
         cosa = param * cos (l[i].angle);
         sina = param * sin (l[i].angle);
         l[i].x = ((float) rx / 2.0f) + cosa;
