@@ -26,7 +26,7 @@
  * is #TRUE, generates an element message named
  * <classname>&quot;level&quot;</classname>:
  * after each interval of time given by the #GstLevel:interval property.
- * The message's structure contains four fields:
+ * The message's structure contains these fields:
  * <itemizedlist>
  * <listitem>
  *   <para>
@@ -108,17 +108,13 @@
 #include <math.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
-/*#include <liboil/liboil.h>*/
 
 #include "gstlevel.h"
 
 GST_DEBUG_CATEGORY_STATIC (level_debug);
 #define GST_CAT_DEFAULT level_debug
 
-static const GstElementDetails level_details = GST_ELEMENT_DETAILS ("Level",
-    "Filter/Analyzer/Audio",
-    "RMS/Peak/Decaying Peak Level messager for audio/raw",
-    "Thomas Vander Stichele <thomas at apestaart dot org>");
+#define EPSILON 1e-35f
 
 static GstStaticPadTemplate sink_template_factory =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -184,11 +180,14 @@ gst_level_base_init (gpointer g_class)
 {
   GstElementClass *element_class = g_class;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template_factory));
-  gst_element_class_set_details (element_class, &level_details);
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &src_template_factory);
+  gst_element_class_set_details_simple (element_class, "Level",
+      "Filter/Analyzer/Audio",
+      "RMS/Peak/Decaying Peak Level messager for audio/raw",
+      "Thomas Vander Stichele <thomas at apestaart dot org>");
 }
 
 static void
@@ -202,21 +201,23 @@ gst_level_class_init (GstLevelClass * klass)
   gobject_class->finalize = gst_level_finalize;
 
   g_object_class_install_property (gobject_class, PROP_SIGNAL_LEVEL,
-      g_param_spec_boolean ("message", "mesage",
+      g_param_spec_boolean ("message", "message",
           "Post a level message for each passed interval",
-          TRUE, G_PARAM_READWRITE));
+          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_SIGNAL_INTERVAL,
       g_param_spec_uint64 ("interval", "Interval",
           "Interval of time between message posts (in nanoseconds)",
-          1, G_MAXUINT64, GST_SECOND / 10, G_PARAM_READWRITE));
+          1, G_MAXUINT64, GST_SECOND / 10,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PEAK_TTL,
       g_param_spec_uint64 ("peak-ttl", "Peak TTL",
           "Time To Live of decay peak before it falls back (in nanoseconds)",
-          0, G_MAXUINT64, GST_SECOND / 10 * 3, G_PARAM_READWRITE));
+          0, G_MAXUINT64, GST_SECOND / 10 * 3,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_PEAK_FALLOFF,
       g_param_spec_double ("peak-falloff", "Peak Falloff",
           "Decay rate of decay peak after TTL (in dB/sec)",
-          0.0, G_MAXDOUBLE, 10.0, G_PARAM_READWRITE));
+          0.0, G_MAXDOUBLE, 10.0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   GST_DEBUG_CATEGORY_INIT (level_debug, "level", 0, "Level calculation");
 
@@ -281,6 +282,10 @@ gst_level_set_property (GObject * object, guint prop_id,
       break;
     case PROP_SIGNAL_INTERVAL:
       filter->interval = g_value_get_uint64 (value);
+      if (filter->rate) {
+        filter->interval_frames =
+            GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate);
+      }
       break;
     case PROP_PEAK_TTL:
       filter->decay_peak_ttl =
@@ -427,7 +432,7 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   GstLevel *filter = GST_LEVEL (trans);
   const gchar *mimetype;
   GstStructure *structure;
-  int i;
+  gint i;
 
   structure = gst_caps_get_structure (in, 0);
   filter->rate = structure_get_int (structure, "rate");
@@ -469,20 +474,23 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   g_free (filter->decay_peak);
   g_free (filter->decay_peak_base);
   g_free (filter->decay_peak_age);
-  filter->CS = g_new (double, filter->channels);
-  filter->peak = g_new (double, filter->channels);
-  filter->last_peak = g_new (double, filter->channels);
-  filter->decay_peak = g_new (double, filter->channels);
-  filter->decay_peak_base = g_new (double, filter->channels);
+  filter->CS = g_new (gdouble, filter->channels);
+  filter->peak = g_new (gdouble, filter->channels);
+  filter->last_peak = g_new (gdouble, filter->channels);
+  filter->decay_peak = g_new (gdouble, filter->channels);
+  filter->decay_peak_base = g_new (gdouble, filter->channels);
 
   filter->decay_peak_age = g_new (GstClockTime, filter->channels);
 
   for (i = 0; i < filter->channels; ++i) {
     filter->CS[i] = filter->peak[i] = filter->last_peak[i] =
         filter->decay_peak[i] = filter->decay_peak_base[i] = 0.0;
-    filter->decay_peak_age[i] = G_GINT64_CONSTANT (0);
+    filter->decay_peak_age[i] = G_GUINT64_CONSTANT (0);
   }
 
+  filter->interval_frames =
+      GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate);
+
   return TRUE;
 }
 
@@ -562,11 +570,12 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 {
   GstLevel *filter;
   guint8 *in_data;
-  double CS;
+  gdouble CS;
   guint i;
   guint num_frames = 0;
   guint num_int_samples = 0;    /* number of interleaved samples
                                  * ie. total count for all channels combined */
+  GstClockTimeDiff falloff_time;
 
   filter = GST_LEVEL (trans);
 
@@ -605,16 +614,15 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
       filter->last_peak[i] = filter->peak[i];
 
     /* make decay peak fall off if too old */
-    if (gst_guint64_to_gdouble (filter->decay_peak_age[i]) >
-        filter->decay_peak_ttl) {
-      double falloff_dB;
-      double falloff;
-      GstClockTimeDiff falloff_time;
-      double length;            /* length of falloff time in seconds */
+    falloff_time =
+        GST_CLOCK_DIFF (gst_gdouble_to_guint64 (filter->decay_peak_ttl),
+        filter->decay_peak_age[i]);
+    if (falloff_time > 0) {
+      gdouble falloff_dB;
+      gdouble falloff;
+      gdouble length;           /* length of falloff time in seconds */
 
-      falloff_time = GST_CLOCK_DIFF (filter->decay_peak_ttl,
-          gst_guint64_to_gdouble (filter->decay_peak_age[i]));
-      length = (gdouble) falloff_time / GST_SECOND;
+      length = (gdouble) falloff_time / (gdouble) GST_SECOND;
       falloff_dB = filter->decay_peak_falloff * length;
       falloff = pow (10, falloff_dB / -20.0);
 
@@ -648,8 +656,7 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
   filter->num_frames += num_frames;
 
   /* do we need to message ? */
-  if (filter->num_frames >=
-      GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate)) {
+  if (filter->num_frames >= filter->interval_frames) {
     if (filter->message) {
       GstMessage *m;
       GstClockTime duration =
@@ -662,8 +669,8 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
           GST_TIME_ARGS (filter->message_ts), filter->num_frames);
 
       for (i = 0; i < filter->channels; ++i) {
-        double RMS;
-        double RMSdB, lastdB, decaydB;
+        gdouble RMS;
+        gdouble RMSdB, lastdB, decaydB;
 
         RMS = sqrt (filter->CS[i] / filter->num_frames);
         GST_LOG_OBJECT (filter,
@@ -673,10 +680,10 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
             "message: last_peak: %f, decay_peak: %f",
             filter->last_peak[i], filter->decay_peak[i]);
         /* RMS values are calculated in amplitude, so 20 * log 10 */
-        RMSdB = 20 * log10 (RMS);
+        RMSdB = 20 * log10 (RMS + EPSILON);
         /* peak values are square sums, ie. power, so 10 * log 10 */
-        lastdB = 10 * log10 (filter->last_peak[i]);
-        decaydB = 10 * log10 (filter->decay_peak[i]);
+        lastdB = 10 * log10 (filter->last_peak[i] + EPSILON);
+        decaydB = 10 * log10 (filter->decay_peak[i] + EPSILON);
 
         if (filter->decay_peak[i] < filter->last_peak[i]) {
           /* this can happen in certain cases, for example when
