@@ -26,6 +26,13 @@
 
 #include "ebml-ids.h"
 
+/*
+ * EBML DocType.
+ */
+
+#define GST_MATROSKA_DOCTYPE_MATROSKA              "matroska"
+#define GST_MATROSKA_DOCTYPE_WEBM                  "webm"
+
 /*
  * Matroska element IDs. max. 32-bit.
  */
@@ -337,6 +344,7 @@
 #define GST_MATROSKA_CODEC_ID_VIDEO_QUICKTIME    "V_QUICKTIME"
 #define GST_MATROSKA_CODEC_ID_VIDEO_SNOW         "V_SNOW"
 #define GST_MATROSKA_CODEC_ID_VIDEO_DIRAC        "V_DIRAC"
+#define GST_MATROSKA_CODEC_ID_VIDEO_VP8          "V_VP8"
 
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L1       "A_MPEG/L1"
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L2       "A_MPEG/L2"
@@ -351,6 +359,8 @@
 #define GST_MATROSKA_CODEC_ID_AUDIO_DTS            "A_DTS"
 #define GST_MATROSKA_CODEC_ID_AUDIO_VORBIS         "A_VORBIS"
 #define GST_MATROSKA_CODEC_ID_AUDIO_FLAC           "A_FLAC"
+/* FIXME: not yet in the spec */
+#define GST_MATROSKA_CODEC_ID_AUDIO_SPEEX          "A_SPEEX"
 #define GST_MATROSKA_CODEC_ID_AUDIO_ACM            "A_MS/ACM"
 #define GST_MATROSKA_CODEC_ID_AUDIO_TTA            "A_TTA1"
 #define GST_MATROSKA_CODEC_ID_AUDIO_WAVPACK4       "A_WAVPACK4"
@@ -375,7 +385,9 @@
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_ASS       "S_TEXT/ASS" 
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_USF       "S_TEXT/USF"
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_VOBSUB    "S_VOBSUB"
+#define GST_MATROSKA_CODEC_ID_SUBTITLE_HDMVPGS   "S_HDMV/PGS"
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_BMP       "S_IMAGE/BMP"
+#define GST_MATROSKA_CODEC_ID_SUBTITLE_KATE      "S_KATE"
 
 /*
  * Matroska tags. Strings.
@@ -383,6 +395,7 @@
 
 #define GST_MATROSKA_TAG_ID_TITLE    "TITLE"
 #define GST_MATROSKA_TAG_ID_AUTHOR   "AUTHOR"
+#define GST_MATROSKA_TAG_ID_ARTIST   "ARTIST"
 #define GST_MATROSKA_TAG_ID_ALBUM    "ALBUM"
 #define GST_MATROSKA_TAG_ID_COMMENTS "COMMENTS"
 #define GST_MATROSKA_TAG_ID_BITSPS   "BITSPS"
@@ -462,12 +475,20 @@ struct _GstMatroskaTrackContext {
   GstCaps      *caps;
   guint         index;
   GstFlowReturn last_flow;
+  /* reverse playback */
+  GstClockTime  from_time;
+  gint64                   from_offset;
+  gint64                   to_offset;
+
+  GArray       *index_table;
+
+  gint          index_writer_id;
 
   /* some often-used info */
   gchar        *codec_id, *codec_name, *name, *language;
-  gpointer      codec_priv;
+  guint8       *codec_priv;
   guint         codec_priv_size;
-  gpointer      codec_state;
+  guint8       *codec_state;
   guint         codec_state_size;
   GstMatroskaTrackType type;
   guint         uid, num;
@@ -488,6 +509,11 @@ struct _GstMatroskaTrackContext {
    * testing for time == 0 is not enough to detect that. Used by demuxer */
   gboolean      send_flac_headers;
 
+  /* Special flag for Speex, for which we need to reconstruct the header
+   * buffer from the codec_priv data before sending any data, and just
+   * testing for time == 0 is not enough to detect that. Used by demuxer */
+  gboolean      send_speex_headers;
+
   /* Special flag for VobSub, for which we have to send colour table info
    * (if available) first before sending any data, and just testing
    * for time == 0 is not enough to detect that. Used by demuxer */
@@ -508,6 +534,12 @@ struct _GstMatroskaTrackContext {
   /* A GArray of GstMatroskaTrackEncoding structures which contain the
    * encoding (compression/encryption) settings for this track, if any */
   GArray       *encodings;
+
+  /* Whether the stream is EOS */
+  gboolean      eos;
+
+  /* any alignment we need our output buffers to have */
+  gint          alignment;
 };
 
 typedef struct _GstMatroskaTrackVideoContext {
@@ -519,6 +551,9 @@ typedef struct _GstMatroskaTrackVideoContext {
   GstMatroskaAspectRatioMode asr_mode;
   guint32       fourcc;
 
+  /* QoS */
+  GstClockTime  earliest_time;
+
   GstBuffer     *dirac_unit;
 } GstMatroskaTrackVideoContext;
 
@@ -534,7 +569,10 @@ typedef struct _GstMatroskaTrackSubtitleContext {
   GstMatroskaTrackContext parent;
 
   gboolean    check_utf8;     /* buffers should be valid UTF-8 */
+  gboolean    check_markup;   /* check if buffers contain markup
+                               * or plaintext and escape characters */
   gboolean    invalid_utf8;   /* work around broken files      */
+  gboolean    seen_markup_tag;  /* markup found in text */
 } GstMatroskaTrackSubtitleContext;
 
 typedef struct _GstMatroskaIndex {
