@@ -42,20 +42,6 @@ G_BEGIN_DECLS
 #define GST_IS_MATROSKA_MUX_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MATROSKA_MUX))
 
-typedef struct _BITMAPINFOHEADER {
-  guint32 bi_size;
-  guint32 bi_width;
-  guint32 bi_height;
-  guint16 bi_planes;
-  guint16 bi_bit_count;
-  guint32 bi_compression;
-  guint32 bi_size_image;
-  guint32 bi_x_pels_per_meter;
-  guint32 bi_y_pels_per_meter;
-  guint32 bi_clr_used;
-  guint32 bi_clr_important;
-} BITMAPINFOHEADER;
-
 typedef enum {
   GST_MATROSKA_MUX_STATE_START,
   GST_MATROSKA_MUX_STATE_HEADER,
@@ -78,12 +64,15 @@ typedef struct
   guint64 duration;
   GstClockTime start_ts;
   GstClockTime end_ts;    /* last timestamp + (if available) duration */
+  guint64 default_duration_scaled;
 }
 GstMatroskaPad;
 
 
 typedef struct _GstMatroskaMux {
   GstElement     element;
+  
+  /* < private > */
 
   /* pads */
   GstPad        *srcpad;
@@ -97,8 +86,11 @@ typedef struct _GstMatroskaMux {
   /* Application name (for the writing application header element) */
   gchar          *writing_app;
 
-  /* Matroska version. */
-  guint          matroska_version;
+  /* EBML DocType. */
+  const gchar    *doctype;
+
+  /* DocType version. */
+  guint          doctype_version;
 
   /* state */
   GstMatroskaMuxState state;
@@ -106,9 +98,13 @@ typedef struct _GstMatroskaMux {
   /* a cue (index) table */
   GstMatroskaIndex *index;
   guint          num_indexes;
-
+  GstClockTimeDiff min_index_interval;
+  gboolean       streamable;
+ 
   /* timescale in the file */
   guint64        time_scale;
+  /* based on timescale, limit of nanoseconds you can have in a cluster */ 
+  guint64        max_cluster_duration;
 
   /* length, position (time, ns) */
   guint64        duration;
@@ -127,23 +123,19 @@ typedef struct _GstMatroskaMux {
   /* current cluster */
   guint64        cluster,
                  cluster_time,
-                 cluster_pos;
+                 cluster_pos,
+		 prev_cluster_size;
 
-  /* meta-seek info */
-  GstMatroskaMetaSeekIndex *meta_index;
-  guint          num_meta_indexes;
+  /* GstForceKeyUnit event */
+  GstEvent       *force_key_unit_event;
 
-  /* tags */
-  GstTagList     *tags;
 } GstMatroskaMux;
 
 typedef struct _GstMatroskaMuxClass {
   GstElementClass parent;
 } GstMatroskaMuxClass;
 
-GType    gst_matroska_mux_get_type    (void);
-
-gboolean gst_matroska_mux_plugin_init (GstPlugin *plugin);
+GType   gst_matroska_mux_get_type (void);
 
 G_END_DECLS
 
