@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -47,13 +47,6 @@ typedef struct _GstAsteriskH263Header
 #define GST_ASTERISKH263_HEADER_TIMESTAMP(buf) (((GstAsteriskH263Header *)(GST_BUFFER_DATA (buf)))->timestamp)
 #define GST_ASTERISKH263_HEADER_LENGTH(buf) (((GstAsteriskH263Header *)(GST_BUFFER_DATA (buf)))->length)
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_h263p_depaydetails =
-GST_ELEMENT_DETAILS ("RTP packet parser",
-    "Codec/Depayloader/Network",
-    "Extracts H263 video from RTP and encodes in Asterisk H263 format",
-    "Neil Stratford <neils@vipadia.com>");
-
 static GstStaticPadTemplate gst_asteriskh263_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -71,9 +64,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
         "clock-rate = (int) 90000, " "encoding-name = (string) \"H263-1998\"")
     );
 
-static void gst_asteriskh263_class_init (GstAsteriskh263Class * klass);
-static void gst_asteriskh263_base_init (GstAsteriskh263Class * klass);
-static void gst_asteriskh263_init (GstAsteriskh263 * asteriskh263);
 static void gst_asteriskh263_finalize (GObject * object);
 
 static GstFlowReturn gst_asteriskh263_chain (GstPad * pad, GstBuffer * buffer);
@@ -81,44 +71,23 @@ static GstFlowReturn gst_asteriskh263_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn gst_asteriskh263_change_state (GstElement *
     element, GstStateChange transition);
 
-static GstElementClass *parent_class = NULL;
-
-static GType
-gst_asteriskh263_get_type (void)
-{
-  static GType asteriskh263_type = 0;
-
-  if (!asteriskh263_type) {
-    static const GTypeInfo asteriskh263_info = {
-      sizeof (GstAsteriskh263Class),
-      (GBaseInitFunc) gst_asteriskh263_base_init,
-      NULL,
-      (GClassInitFunc) gst_asteriskh263_class_init,
-      NULL,
-      NULL,
-      sizeof (GstAsteriskh263),
-      0,
-      (GInstanceInitFunc) gst_asteriskh263_init,
-    };
-
-    asteriskh263_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstAsteriskh263",
-        &asteriskh263_info, 0);
-  }
-  return asteriskh263_type;
-}
+GST_BOILERPLATE (GstAsteriskh263, gst_asteriskh263, GstElement,
+    GST_TYPE_ELEMENT);
 
 static void
-gst_asteriskh263_base_init (GstAsteriskh263Class * klass)
+gst_asteriskh263_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_asteriskh263_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_asteriskh263_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_asteriskh263_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_asteriskh263_sink_template);
 
-  gst_element_class_set_details (element_class, &gst_rtp_h263p_depaydetails);
+  gst_element_class_set_details_simple (element_class,
+      "RTP Asterisk H263 depayloader", "Codec/Depayloader/Network/RTP",
+      "Extracts H263 video from RTP and encodes in Asterisk H263 format",
+      "Neil Stratford <neils@vipadia.com>");
 }
 
 static void
@@ -130,15 +99,14 @@ gst_asteriskh263_class_init (GstAsteriskh263Class * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->finalize = gst_asteriskh263_finalize;
 
   gstelement_class->change_state = gst_asteriskh263_change_state;
 }
 
 static void
-gst_asteriskh263_init (GstAsteriskh263 * asteriskh263)
+gst_asteriskh263_init (GstAsteriskh263 * asteriskh263,
+    GstAsteriskh263Class * klass)
 {
   asteriskh263->srcpad =
       gst_pad_new_from_static_template (&gst_asteriskh263_src_template, "src");
@@ -204,8 +172,8 @@ gst_asteriskh263_chain (GstPad * pad, GstBuffer * buf)
     samples = timestamp - asteriskh263->lastts;
     asteriskh263->lastts = timestamp;
 
-    GST_ASTERISKH263_HEADER_TIMESTAMP (outbuf) = htonl (samples);
-    GST_ASTERISKH263_HEADER_LENGTH (outbuf) = htons (asterisk_len);
+    GST_ASTERISKH263_HEADER_TIMESTAMP (outbuf) = g_htonl (samples);
+    GST_ASTERISKH263_HEADER_LENGTH (outbuf) = g_htons (asterisk_len);
 
     /* copy the data into place */
     memcpy (GST_BUFFER_DATA (outbuf) + GST_ASTERISKH263_HEADER_LEN, payload,
@@ -263,5 +231,5 @@ gboolean
 gst_asteriskh263_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "asteriskh263",
-      GST_RANK_MARGINAL, GST_TYPE_ASTERISK_H263);
+      GST_RANK_SECONDARY, GST_TYPE_ASTERISK_H263);
 }
