@@ -32,10 +32,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpamrpay_debug);
 
 /* references:
  *
- * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File 
- *    Storage Format for the Adaptive Multi-Rate (AMR) and Adaptive 
+ * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File
+ *    Storage Format for the Adaptive Multi-Rate (AMR) and Adaptive
  *    Multi-Rate Wideband (AMR-WB) Audio Codecs.
- *    
+ *
  * ETSI TS 126 201 V6.0.0 (2004-12) - Digital cellular telecommunications system (Phase 2+);
  *                 Universal Mobile Telecommunications System (UMTS);
  *                          AMR speech codec, wideband;
@@ -43,13 +43,6 @@ GST_DEBUG_CATEGORY_STATIC (rtpamrpay_debug);
  *                    (3GPP TS 26.201 version 6.0.0 Release 6)
  */
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_amrpay_details =
-GST_ELEMENT_DETAILS ("RTP AMR payloader",
-    "Codec/Payloader/Network",
-    "Payload-encode AMR or AMR-WB audio into RTP packets (RFC 3267)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_amr_pay_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -97,6 +90,9 @@ static gboolean gst_rtp_amr_pay_setcaps (GstBaseRTPPayload * basepayload,
 static GstFlowReturn gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
 
+static GstStateChangeReturn
+gst_rtp_amr_pay_change_state (GstElement * element, GstStateChange transition);
+
 GST_BOILERPLATE (GstRtpAMRPay, gst_rtp_amr_pay, GstBaseRTPPayload,
     GST_TYPE_BASE_RTP_PAYLOAD);
 
@@ -105,26 +101,27 @@ gst_rtp_amr_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_amr_pay_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_amr_pay_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_amr_pay_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_amr_pay_sink_template);
 
-  gst_element_class_set_details (element_class, &gst_rtp_amrpay_details);
+  gst_element_class_set_details_simple (element_class, "RTP AMR payloader",
+      "Codec/Payloader/Network/RTP",
+      "Payload-encode AMR or AMR-WB audio into RTP packets (RFC 3267)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
 gst_rtp_amr_pay_class_init (GstRtpAMRPayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
+  GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
-  gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
+  gstelement_class->change_state = gst_rtp_amr_pay_change_state;
 
-  parent_class = g_type_class_peek_parent (klass);
+  gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
   gstbasertppayload_class->set_caps = gst_rtp_amr_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_amr_pay_handle_buffer;
@@ -139,10 +136,19 @@ gst_rtp_amr_pay_init (GstRtpAMRPay * rtpamrpay, GstRtpAMRPayClass * klass)
   /* needed because of GST_BOILERPLATE */
 }
 
+static void
+gst_rtp_amr_pay_reset (GstRtpAMRPay * pay)
+{
+  pay->next_rtp_time = 0;
+  pay->first_ts = GST_CLOCK_TIME_NONE;
+  pay->first_rtp_time = 0;
+}
+
 static gboolean
 gst_rtp_amr_pay_setcaps (GstBaseRTPPayload * basepayload, GstCaps * caps)
 {
   GstRtpAMRPay *rtpamrpay;
+  gboolean res;
   const GstStructure *s;
   const gchar *str;
 
@@ -166,17 +172,17 @@ gst_rtp_amr_pay_setcaps (GstBaseRTPPayload * basepayload, GstCaps * caps)
     gst_basertppayload_set_options (basepayload, "audio", TRUE, "AMR-WB",
         16000);
 
-  gst_basertppayload_set_outcaps (basepayload,
+  res = gst_basertppayload_set_outcaps (basepayload,
       "encoding-params", G_TYPE_STRING, "1", "octet-align", G_TYPE_STRING, "1",
-      /* don't set the defaults 
-       * 
+      /* don't set the defaults
+       *
        * "crc", G_TYPE_STRING, "0",
        * "robust-sorting", G_TYPE_STRING, "0",
-       * "interleaving", G_TYPE_STRING, "0", 
+       * "interleaving", G_TYPE_STRING, "0",
        */
       NULL);
 
-  return TRUE;
+  return res;
 
   /* ERRORS */
 wrong_type:
@@ -187,14 +193,38 @@ wrong_type:
   }
 }
 
+static void
+gst_rtp_amr_pay_recalc_rtp_time (GstRtpAMRPay * rtpamrpay,
+    GstClockTime timestamp)
+{
+  /* re-sync rtp time */
+  if (GST_CLOCK_TIME_IS_VALID (rtpamrpay->first_ts) &&
+      GST_CLOCK_TIME_IS_VALID (timestamp) && timestamp >= rtpamrpay->first_ts) {
+    GstClockTime diff;
+    guint32 rtpdiff;
+
+    /* interpolate to reproduce gap from start, rather than intermediate
+     * intervals to avoid roundup accumulation errors */
+    diff = timestamp - rtpamrpay->first_ts;
+    rtpdiff = ((diff / GST_MSECOND) * 8) <<
+        (rtpamrpay->mode == GST_RTP_AMR_P_MODE_WB);
+    rtpamrpay->next_rtp_time = rtpamrpay->first_rtp_time + rtpdiff;
+    GST_DEBUG_OBJECT (rtpamrpay,
+        "elapsed time %" GST_TIME_FORMAT ", rtp %" G_GUINT32_FORMAT ", "
+        "new offset %" G_GUINT32_FORMAT, GST_TIME_ARGS (diff), rtpdiff,
+        rtpamrpay->next_rtp_time);
+  }
+}
+
 /* -1 is invalid */
-static gint nb_frame_size[16] = {
+static const gint nb_frame_size[16] = {
   12, 13, 15, 17, 19, 20, 26, 31,
   5, -1, -1, -1, -1, -1, -1, 0
 };
-static gint wb_frame_size[16] = {
+
+static const gint wb_frame_size[16] = {
   17, 23, 32, 36, 40, 46, 50, 58,
-  60, -1, -1, -1, -1, -1, -1, 0
+  60, 5, -1, -1, -1, -1, -1, 0
 };
 
 static GstFlowReturn
@@ -202,6 +232,7 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
     GstBuffer * buffer)
 {
   GstRtpAMRPay *rtpamrpay;
+  const gint *frame_size;
   GstFlowReturn ret;
   guint size, payload_len;
   GstBuffer *outbuf;
@@ -210,8 +241,7 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   guint packet_len, mtu;
   gint i, num_packets, num_nonempty_packets;
   gint amr_len;
-  gint *frame_size;
-  gboolean discont;
+  gboolean sid = FALSE;
 
   rtpamrpay = GST_RTP_AMR_PAY (basepayload);
   mtu = GST_BASE_RTP_PAYLOAD_MTU (rtpamrpay);
@@ -220,7 +250,6 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   data = GST_BUFFER_DATA (buffer);
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
   duration = GST_BUFFER_DURATION (buffer);
-  discont = GST_BUFFER_IS_DISCONT (buffer);
 
   /* setup frame size pointer */
   if (rtpamrpay->mode == GST_RTP_AMR_P_MODE_NB)
@@ -230,7 +259,7 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   GST_DEBUG_OBJECT (basepayload, "got %d bytes", size);
 
-  /* FIXME, only 
+  /* FIXME, only
    * octet aligned, no interleaving, single channel, no CRC,
    * no robust-sorting. To fix this you need to implement the downstream
    * negotiation function. */
@@ -244,11 +273,14 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
     FT = (data[i] & 0x78) >> 3;
 
     fr_size = frame_size[FT];
-    GST_DEBUG_OBJECT (basepayload, "frame size %d", fr_size);
+    GST_DEBUG_OBJECT (basepayload, "frame type %d, frame size %d", FT, fr_size);
     /* FIXME, we don't handle this yet.. */
     if (fr_size <= 0)
       goto wrong_size;
 
+    if (fr_size == 5)
+      sid = TRUE;
+
     amr_len += fr_size;
     num_nonempty_packets++;
     num_packets++;
@@ -272,24 +304,36 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   /* copy timestamp */
   GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
 
-  /* FIXME: when we do more than one AMR frame per packet, fix this */
   if (duration != GST_CLOCK_TIME_NONE)
     GST_BUFFER_DURATION (outbuf) = duration;
   else {
-    GST_BUFFER_DURATION (outbuf) = 20 * GST_MSECOND;
+    GST_BUFFER_DURATION (outbuf) = num_packets * 20 * GST_MSECOND;
   }
 
-  if (discont) {
+  if (GST_BUFFER_IS_DISCONT (buffer)) {
     GST_DEBUG_OBJECT (basepayload, "discont, setting marker bit");
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
     gst_rtp_buffer_set_marker (outbuf, TRUE);
-    discont = FALSE;
+    gst_rtp_amr_pay_recalc_rtp_time (rtpamrpay, timestamp);
   }
 
+  if (G_UNLIKELY (sid)) {
+    gst_rtp_amr_pay_recalc_rtp_time (rtpamrpay, timestamp);
+  }
+
+  /* perfect rtptime */
+  if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (rtpamrpay->first_ts))) {
+    rtpamrpay->first_ts = timestamp;
+    rtpamrpay->first_rtp_time = rtpamrpay->next_rtp_time;
+  }
+  GST_BUFFER_OFFSET (outbuf) = rtpamrpay->next_rtp_time;
+  rtpamrpay->next_rtp_time +=
+      (num_packets * 160) << (rtpamrpay->mode == GST_RTP_AMR_P_MODE_WB);
+
   /* get payload, this is now writable */
   payload = gst_rtp_buffer_get_payload (outbuf);
 
-  /*   0 1 2 3 4 5 6 7 
+  /*   0 1 2 3 4 5 6 7
    *  +-+-+-+-+-+-+-+-+
    *  |  CMR  |R|R|R|R|
    *  +-+-+-+-+-+-+-+-+
@@ -362,9 +406,34 @@ too_big:
   }
 }
 
+static GstStateChangeReturn
+gst_rtp_amr_pay_change_state (GstElement * element, GstStateChange transition)
+{
+  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
+
+  /* handle upwards state changes here */
+  switch (transition) {
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  /* handle downwards state changes */
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_rtp_amr_pay_reset (GST_RTP_AMR_PAY (element));
+      break;
+    default:
+      break;
+  }
+
+  return ret;
+}
+
 gboolean
 gst_rtp_amr_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpamrpay",
-      GST_RANK_NONE, GST_TYPE_RTP_AMR_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_AMR_PAY);
 }
