@@ -27,12 +27,8 @@
 
 #include "gstrtpmpapay.h"
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_mpapay_details =
-GST_ELEMENT_DETAILS ("RTP MPEG audio payloader",
-    "Codec/Payloader/Network",
-    "Payload MPEG audio as RTP packets (RFC 2038)",
-    "Wim Taymans <wim.taymans@gmail.com>");
+GST_DEBUG_CATEGORY_STATIC (rtpmpapay_debug);
+#define GST_CAT_DEFAULT (rtpmpapay_debug)
 
 static GstStaticPadTemplate gst_rtp_mpa_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -55,54 +51,34 @@ static GstStaticPadTemplate gst_rtp_mpa_pay_src_template =
         "clock-rate = (int) 90000, " "encoding-name = (string) \"MPA\"")
     );
 
-static void gst_rtp_mpa_pay_class_init (GstRtpMPAPayClass * klass);
-static void gst_rtp_mpa_pay_base_init (GstRtpMPAPayClass * klass);
-static void gst_rtp_mpa_pay_init (GstRtpMPAPay * rtpmpapay);
 static void gst_rtp_mpa_pay_finalize (GObject * object);
 
+static GstStateChangeReturn gst_rtp_mpa_pay_change_state (GstElement * element,
+    GstStateChange transition);
+
 static gboolean gst_rtp_mpa_pay_setcaps (GstBaseRTPPayload * payload,
     GstCaps * caps);
+static gboolean gst_rtp_mpa_pay_handle_event (GstPad * pad, GstEvent * event);
+static GstFlowReturn gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay);
 static GstFlowReturn gst_rtp_mpa_pay_handle_buffer (GstBaseRTPPayload * payload,
     GstBuffer * buffer);
 
-static GstBaseRTPPayloadClass *parent_class = NULL;
+GST_BOILERPLATE (GstRtpMPAPay, gst_rtp_mpa_pay, GstBaseRTPPayload,
+    GST_TYPE_BASE_RTP_PAYLOAD)
 
-static GType
-gst_rtp_mpa_pay_get_type (void)
-{
-  static GType rtpmpapay_type = 0;
-
-  if (!rtpmpapay_type) {
-    static const GTypeInfo rtpmpapay_info = {
-      sizeof (GstRtpMPAPayClass),
-      (GBaseInitFunc) gst_rtp_mpa_pay_base_init,
-      NULL,
-      (GClassInitFunc) gst_rtp_mpa_pay_class_init,
-      NULL,
-      NULL,
-      sizeof (GstRtpMPAPay),
-      0,
-      (GInstanceInitFunc) gst_rtp_mpa_pay_init,
-    };
-
-    rtpmpapay_type =
-        g_type_register_static (GST_TYPE_BASE_RTP_PAYLOAD, "GstRtpMPAPay",
-        &rtpmpapay_info, 0);
-  }
-  return rtpmpapay_type;
-}
-
-static void
-gst_rtp_mpa_pay_base_init (GstRtpMPAPayClass * klass)
+     static void gst_rtp_mpa_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_mpa_pay_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_mpa_pay_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_mpa_pay_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_mpa_pay_sink_template);
 
-  gst_element_class_set_details (element_class, &gst_rtp_mpapay_details);
+  gst_element_class_set_details_simple (element_class,
+      "RTP MPEG audio payloader", "Codec/Payloader/Network/RTP",
+      "Payload MPEG audio as RTP packets (RFC 2038)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -116,16 +92,20 @@ gst_rtp_mpa_pay_class_init (GstRtpMPAPayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->finalize = gst_rtp_mpa_pay_finalize;
 
+  gstelement_class->change_state = gst_rtp_mpa_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_mpa_pay_setcaps;
+  gstbasertppayload_class->handle_event = gst_rtp_mpa_pay_handle_event;
   gstbasertppayload_class->handle_buffer = gst_rtp_mpa_pay_handle_buffer;
+
+  GST_DEBUG_CATEGORY_INIT (rtpmpapay_debug, "rtpmpapay", 0,
+      "MPEG Audio RTP Depayloader");
 }
 
 static void
-gst_rtp_mpa_pay_init (GstRtpMPAPay * rtpmpapay)
+gst_rtp_mpa_pay_init (GstRtpMPAPay * rtpmpapay, GstRtpMPAPayClass * klass)
 {
   rtpmpapay->adapter = gst_adapter_new ();
 }
@@ -143,6 +123,15 @@ gst_rtp_mpa_pay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gst_rtp_mpa_pay_reset (GstRtpMPAPay * pay)
+{
+  pay->first_ts = -1;
+  pay->duration = 0;
+  gst_adapter_clear (pay->adapter);
+  GST_DEBUG_OBJECT (pay, "reset depayloader");
+}
+
 static gboolean
 gst_rtp_mpa_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
 {
@@ -154,6 +143,31 @@ gst_rtp_mpa_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
   return res;
 }
 
+static gboolean
+gst_rtp_mpa_pay_handle_event (GstPad * pad, GstEvent * event)
+{
+  GstRtpMPAPay *rtpmpapay;
+
+  rtpmpapay = GST_RTP_MPA_PAY (gst_pad_get_parent (pad));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      /* make sure we push the last packets in the adapter on EOS */
+      gst_rtp_mpa_pay_flush (rtpmpapay);
+      break;
+    case GST_EVENT_FLUSH_STOP:
+      gst_rtp_mpa_pay_reset (rtpmpapay);
+      break;
+    default:
+      break;
+  }
+
+  gst_object_unref (rtpmpapay);
+
+  /* FALSE to let the parent handle the event as well */
+  return FALSE;
+}
+
 static GstFlowReturn
 gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay)
 {
@@ -179,7 +193,7 @@ gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay)
     guint payload_len;
     guint packet_len;
 
-    /* this will be the total lenght of the packet */
+    /* this will be the total length of the packet */
     packet_len = gst_rtp_buffer_calc_packet_len (4 + avail, 0, 0);
 
     /* fill one MTU or all available bytes */
@@ -200,7 +214,7 @@ gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay)
      *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
      * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      * |             MBZ               |          Frag_offset          |
-     * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
+     * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      */
     payload = gst_rtp_buffer_get_payload (outbuf);
     payload[0] = 0;
@@ -234,20 +248,22 @@ gst_rtp_mpa_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   GstFlowReturn ret;
   guint size, avail;
   guint packet_len;
-  GstClockTime duration;
+  GstClockTime duration, timestamp;
 
   rtpmpapay = GST_RTP_MPA_PAY (basepayload);
 
   size = GST_BUFFER_SIZE (buffer);
   duration = GST_BUFFER_DURATION (buffer);
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
-  avail = gst_adapter_available (rtpmpapay->adapter);
-  if (avail == 0) {
-    rtpmpapay->first_ts = GST_BUFFER_TIMESTAMP (buffer);
-    rtpmpapay->duration = 0;
+  if (GST_BUFFER_IS_DISCONT (buffer)) {
+    GST_DEBUG_OBJECT (rtpmpapay, "DISCONT");
+    gst_rtp_mpa_pay_reset (rtpmpapay);
   }
 
-  /* get packet length of previous data and this new data, 
+  avail = gst_adapter_available (rtpmpapay->adapter);
+
+  /* get packet length of previous data and this new data,
    * payload length includes a 4 byte header */
   packet_len = gst_rtp_buffer_calc_packet_len (4 + avail + size, 0, 0);
 
@@ -256,15 +272,50 @@ gst_rtp_mpa_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   if (gst_basertppayload_is_filled (basepayload,
           packet_len, rtpmpapay->duration + duration)) {
     ret = gst_rtp_mpa_pay_flush (rtpmpapay);
-    rtpmpapay->first_ts = GST_BUFFER_TIMESTAMP (buffer);
-    rtpmpapay->duration = 0;
+    avail = 0;
   } else {
     ret = GST_FLOW_OK;
   }
 
-  gst_adapter_push (rtpmpapay->adapter, buffer);
-  rtpmpapay->duration += duration;
+  if (avail == 0) {
+    GST_DEBUG_OBJECT (rtpmpapay,
+        "first packet, save timestamp %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (timestamp));
+    rtpmpapay->first_ts = timestamp;
+    rtpmpapay->duration = 0;
+  }
 
+  gst_adapter_push (rtpmpapay->adapter, buffer);
+  rtpmpapay->duration = duration;
+
+  return ret;
+}
+
+static GstStateChangeReturn
+gst_rtp_mpa_pay_change_state (GstElement * element, GstStateChange transition)
+{
+  GstRtpMPAPay *rtpmpapay;
+  GstStateChangeReturn ret;
+
+  rtpmpapay = GST_RTP_MPA_PAY (element);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_READY_TO_PAUSED:
+      gst_rtp_mpa_pay_reset (rtpmpapay);
+      break;
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_rtp_mpa_pay_reset (rtpmpapay);
+      break;
+    default:
+      break;
+  }
   return ret;
 }
 
@@ -272,5 +323,5 @@ gboolean
 gst_rtp_mpa_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmpapay",
-      GST_RANK_NONE, GST_TYPE_RTP_MPA_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_MPA_PAY);
 }
