@@ -30,13 +30,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtptheoradepay_debug);
 #define GST_CAT_DEFAULT (rtptheoradepay_debug)
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_theora_depay_details =
-GST_ELEMENT_DETAILS ("RTP Theora depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts Theora video from RTP packets (draft-01 of RFC XXXX)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_theora_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -44,9 +37,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\","
-        /* only support inline delivery */
-        "delivery-method = (string) \"inline\""
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\""
         /* All required parameters 
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
@@ -76,43 +67,42 @@ static gboolean gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload,
     GstCaps * caps);
 static GstBuffer *gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
+static gboolean gst_rtp_theora_depay_packet_lost (GstBaseRTPDepayload *
+    depayload, GstEvent * event);
 
 static void gst_rtp_theora_depay_finalize (GObject * object);
 
-static GstStateChangeReturn gst_rtp_theora_depay_change_state (GstElement *
-    element, GstStateChange transition);
-
 
 static void
 gst_rtp_theora_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_theora_depay_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_theora_depay_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_theora_depay_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_theora_depay_src_template);
 
-  gst_element_class_set_details (element_class, &gst_rtp_theora_depay_details);
+  gst_element_class_set_details_simple (element_class, "RTP Theora depayloader",
+      "Codec/Depayloader/Network/RTP",
+      "Extracts Theora video from RTP packets (draft-01 of RFC XXXX)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
 gst_rtp_theora_depay_class_init (GstRtpTheoraDepayClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   gobject_class->finalize = gst_rtp_theora_depay_finalize;
 
-  gstelement_class->change_state = gst_rtp_theora_depay_change_state;
-
   gstbasertpdepayload_class->process = gst_rtp_theora_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_theora_depay_setcaps;
+  gstbasertpdepayload_class->packet_lost = gst_rtp_theora_depay_packet_lost;
 
   GST_DEBUG_CATEGORY_INIT (rtptheoradepay_debug, "rtptheoradepay", 0,
       "Theora RTP Depayloader");
@@ -124,6 +114,7 @@ gst_rtp_theora_depay_init (GstRtpTheoraDepay * rtptheoradepay,
 {
   rtptheoradepay->adapter = gst_adapter_new ();
 }
+
 static void
 gst_rtp_theora_depay_finalize (GObject * object)
 {
@@ -134,57 +125,9 @@ gst_rtp_theora_depay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static const guint8 a2bin[256] = {
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 62, 64, 64, 64, 63,
-  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 64, 64, 64, 64, 64, 64,
-  64, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 64, 64, 64, 64, 64,
-  64, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64
-};
-
-static guint
-decode_base64 (const gchar * in, guint8 * out)
-{
-  guint8 v1, v2;
-  guint len = 0;
-
-  v1 = a2bin[(gint) * in];
-  while (v1 <= 63) {
-    /* read 4 bytes, write 3 bytes, invalid base64 are zeroes */
-    v2 = a2bin[(gint) * ++in];
-    *out++ = (v1 << 2) | ((v2 & 0x3f) >> 4);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v2 << 4) | ((v1 & 0x3f) >> 2);
-    v2 = (v1 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v1 << 6) | (v2 & 0x3f);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    len += 3;
-  }
-  /* move to '\0' */
-  while (*in != '\0')
-    in++;
-
-  /* subtract padding */
-  while (len > 0 && *--in == '=')
-    len--;
-
-  return len;
-}
-
 static gboolean
 gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
-    const gchar * configuration)
+    GstBuffer * confbuf)
 {
   GstBuffer *buf;
   guint32 num_headers;
@@ -192,12 +135,8 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
   guint size;
   gint i, j;
 
-  /* deserialize base64 to buffer */
-  size = strlen (configuration);
-  GST_DEBUG_OBJECT (rtptheoradepay, "base64 config size %u", size);
-
-  data = g_malloc (size);
-  size = decode_base64 (configuration, data);
+  data = GST_BUFFER_DATA (confbuf);
+  size = GST_BUFFER_SIZE (confbuf);
 
   GST_DEBUG_OBJECT (rtptheoradepay, "config size %u", size);
 
@@ -253,6 +192,7 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
     guint8 n_headers, b;
     GstRtpTheoraConfig *conf;
     guint *h_sizes;
+    guint extra = 1;
 
     if (size < 6)
       goto too_small;
@@ -266,7 +206,10 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
     GST_DEBUG_OBJECT (rtptheoradepay,
         "header %d, ident 0x%08x, length %u, left %u", i, ident, length, size);
 
-    if (size < length)
+    /* FIXME check if we already got this ident */
+
+    /* length might also include count of following size fields */
+    if (size < length && size + 1 != length)
       goto too_small;
 
     /* read header sizes we read 2 sizes, the third size (for which we allocate
@@ -281,6 +224,7 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
           goto too_small;
         b = *data++;
         size--;
+        extra++;
         h_size = (h_size << 7) | (b & 0x7f);
       } while (b & 0x80);
       GST_DEBUG_OBJECT (rtptheoradepay, "headers %d: size: %u", j, h_size);
@@ -299,8 +243,15 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
       guint h_size;
 
       h_size = h_sizes[j];
-      if (size < h_size)
-        goto too_small;
+      if (size < h_size) {
+        if (j != n_headers || size + extra != h_size) {
+          goto too_small;
+        } else {
+          /* otherwise means that overall length field contained total length,
+           * including extra fields */
+          h_size -= extra;
+        }
+      }
 
       GST_DEBUG_OBJECT (rtptheoradepay, "reading header %d, size %u", j,
           h_size);
@@ -323,43 +274,69 @@ too_small:
   }
 }
 
+static gboolean
+gst_rtp_theora_depay_parse_inband_configuration (GstRtpTheoraDepay *
+    rtptheoradepay, guint ident, guint8 * configuration, guint size,
+    guint length)
+{
+  GstBuffer *confbuf;
+  guint8 *conf;
+
+  if (G_UNLIKELY (size < 4))
+    return FALSE;
+
+  /* transform inline to out-of-band and parse that one */
+  confbuf = gst_buffer_new_and_alloc (size + 9);
+  conf = GST_BUFFER_DATA (confbuf);
+  /* 1 header */
+  GST_WRITE_UINT32_BE (conf, 1);
+  /* write Ident */
+  GST_WRITE_UINT24_BE (conf + 4, ident);
+  /* write sort-of-length */
+  GST_WRITE_UINT16_BE (conf + 7, length);
+  /* copy remainder */
+  memcpy (conf + 9, configuration, size);
+
+  return gst_rtp_theora_depay_parse_configuration (rtptheoradepay, confbuf);
+}
+
 static gboolean
 gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstStructure *structure;
   GstRtpTheoraDepay *rtptheoradepay;
   GstCaps *srccaps;
-  const gchar *delivery_method;
   const gchar *configuration;
   gboolean res;
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
+  rtptheoradepay->needs_keyframe = FALSE;
+
   structure = gst_caps_get_structure (caps, 0);
 
-  /* see how the configuration parameters will be transmitted */
-  delivery_method = gst_structure_get_string (structure, "delivery-method");
-  if (delivery_method == NULL)
-    goto no_delivery_method;
-
-  if (!g_ascii_strcasecmp (delivery_method, "inline")) {
-    /* configure string is in the caps */
-  } else if (!g_ascii_strcasecmp (delivery_method, "in_band")) {
-    /* headers will (also) be transmitted in the RTP packets */
-    goto unsupported_delivery_method;
-  } else if (g_str_has_prefix (delivery_method, "out_band/")) {
-    /* some other method of header delivery. */
-    goto unsupported_delivery_method;
-  } else
-    goto unsupported_delivery_method;
-
   /* read and parse configuration string */
   configuration = gst_structure_get_string (structure, "configuration");
-  if (configuration == NULL)
-    goto no_configuration;
+  if (configuration) {
+    GstBuffer *confbuf;
+    guint8 *data;
+    gsize size;
 
-  if (!gst_rtp_theora_depay_parse_configuration (rtptheoradepay, configuration))
-    goto invalid_configuration;
+    /* configure string should be in the caps */
+    if (configuration == NULL)
+      goto no_configuration;
+
+    /* deserialize base64 to buffer */
+    data = g_base64_decode (configuration, &size);
+
+    confbuf = gst_buffer_new ();
+    GST_BUFFER_DATA (confbuf) = data;
+    GST_BUFFER_MALLOCDATA (confbuf) = data;
+    GST_BUFFER_SIZE (confbuf) = size;
+
+    if (!gst_rtp_theora_depay_parse_configuration (rtptheoradepay, confbuf))
+      goto invalid_configuration;
+  }
 
   /* set caps on pad and on header */
   srccaps = gst_caps_new_simple ("video/x-theora", NULL);
@@ -372,17 +349,6 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   return res;
 
   /* ERRORS */
-unsupported_delivery_method:
-  {
-    GST_ERROR_OBJECT (rtptheoradepay,
-        "unsupported delivery-method \"%s\" specified", delivery_method);
-    return FALSE;
-  }
-no_delivery_method:
-  {
-    GST_ERROR_OBJECT (rtptheoradepay, "no delivery-method specified");
-    return FALSE;
-  }
 no_configuration:
   {
     GST_ERROR_OBJECT (rtptheoradepay, "no configuration specified");
@@ -441,7 +407,6 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   guint32 timestamp;
   guint32 header, ident;
   guint8 F, TDT, packets;
-  gboolean free_payload;
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
@@ -454,7 +419,6 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     goto packet_short;
 
   payload = gst_rtp_buffer_get_payload (buf);
-  free_payload = FALSE;
 
   header = GST_READ_UINT32_BE (payload);
   /*
@@ -537,7 +501,7 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     to_free = payload;
   }
 
-  GST_DEBUG_OBJECT (depayload, "assemble done");
+  GST_DEBUG_OBJECT (depayload, "assemble done, payload_len %d", payload_len);
 
   /* we not assembling anymore now */
   rtptheoradepay->assembling = FALSE;
@@ -560,7 +524,7 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
    */
   timestamp = gst_rtp_buffer_get_timestamp (buf);
 
-  while (payload_len > 2) {
+  while (payload_len >= 2) {
     guint16 length;
 
     length = GST_READ_UINT16_BE (payload);
@@ -574,6 +538,15 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     if (G_UNLIKELY (length > payload_len))
       goto length_short;
 
+    /* handle in-band configuration */
+    if (G_UNLIKELY (TDT == 1)) {
+      GST_DEBUG_OBJECT (rtptheoradepay, "in-band configuration");
+      if (!gst_rtp_theora_depay_parse_inband_configuration (rtptheoradepay,
+              ident, payload, payload_len, length))
+        goto invalid_configuration;
+      goto no_output;
+    }
+
     /* create buffer for packet */
     if (G_UNLIKELY (to_free)) {
       outbuf = gst_buffer_new ();
@@ -586,6 +559,9 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
       memcpy (GST_BUFFER_DATA (outbuf), payload, length);
     }
 
+    if (payload_len > 0 && (payload[0] & 0xC0) == 0x0)
+      rtptheoradepay->needs_keyframe = FALSE;
+
     payload += length;
     payload_len -= length;
 
@@ -605,6 +581,9 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   g_free (to_free);
 
+  if (rtptheoradepay->needs_keyframe)
+    goto request_keyframe;
+
   return NULL;
 
 no_output:
@@ -614,15 +593,15 @@ no_output:
   /* ERORRS */
 switch_failed:
   {
-    GST_ELEMENT_ERROR (rtptheoradepay, STREAM, DECODE,
+    GST_ELEMENT_WARNING (rtptheoradepay, STREAM, DECODE,
         (NULL), ("Could not switch codebooks"));
-    return NULL;
+    goto request_config;
   }
 packet_short:
   {
     GST_ELEMENT_WARNING (rtptheoradepay, STREAM, DECODE,
         (NULL), ("Packet was too short (%d < 4)", payload_len));
-    return NULL;
+    goto request_keyframe;
   }
 ignore_reserved:
   {
@@ -633,42 +612,55 @@ length_short:
   {
     GST_ELEMENT_WARNING (rtptheoradepay, STREAM, DECODE,
         (NULL), ("Packet contains invalid data"));
+    goto request_keyframe;
+  }
+invalid_configuration:
+  {
+    /* fatal, as we otherwise risk carrying on without output */
+    GST_ELEMENT_ERROR (rtptheoradepay, STREAM, DECODE,
+        (NULL), ("Packet contains invalid configuration"));
+    goto request_config;
+  }
+request_config:
+  {
+    gst_pad_push_event (GST_BASE_RTP_DEPAYLOAD_SINKPAD (depayload),
+        gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM,
+            gst_structure_new ("GstForceKeyUnit",
+                "all-headers", G_TYPE_BOOLEAN, TRUE, NULL)));
     return NULL;
   }
-}
-
-static GstStateChangeReturn
-gst_rtp_theora_depay_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstRtpTheoraDepay *rtptheoradepay;
-  GstStateChangeReturn ret;
-
-  rtptheoradepay = GST_RTP_THEORA_DEPAY (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      break;
-    default:
-      break;
+request_keyframe:
+  {
+    rtptheoradepay->needs_keyframe = TRUE;
+    gst_pad_push_event (GST_BASE_RTP_DEPAYLOAD_SINKPAD (depayload),
+        gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM,
+            gst_structure_new ("GstForceKeyUnit", NULL)));
+    return NULL;
   }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-  return ret;
 }
 
 gboolean
 gst_rtp_theora_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtptheoradepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_THEORA_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_THEORA_DEPAY);
+}
+
+static gboolean
+gst_rtp_theora_depay_packet_lost (GstBaseRTPDepayload * depayload,
+    GstEvent * event)
+{
+  GstRtpTheoraDepay *rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
+  guint seqnum = 0;
+
+  gst_structure_get_uint (event->structure, "seqnum", &seqnum);
+  GST_LOG_OBJECT (depayload, "Requested keyframe because frame with seqnum %u"
+      " is missing", seqnum);
+  rtptheoradepay->needs_keyframe = TRUE;
+
+  gst_pad_push_event (GST_BASE_RTP_DEPAYLOAD_SINKPAD (depayload),
+      gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM,
+          gst_structure_new ("GstForceKeyUnit", NULL)));
+
+  return TRUE;
 }
