@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,16 +31,9 @@ GST_DEBUG_CATEGORY_STATIC (rtpvorbisdepay_debug);
 #define GST_CAT_DEFAULT (rtpvorbisdepay_debug)
 
 /* references:
- * http://svn.xiph.org/trunk/vorbis/doc/draft-ietf-avt-rtp-vorbis-04.txt
+ * http://www.rfc-editor.org/rfc/rfc5215.txt
  */
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_vorbis_depay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts Vorbis Audio from RTP packets (draft-04 of RFC XXXX)",
-    "Wim Taymans <wim@fluendo.com>");
-
 static GstStaticPadTemplate gst_rtp_vorbis_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -52,13 +45,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
         /* All required parameters 
          *
          * "encoding-params = (string) <num channels>"
-         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } " 
          * "configuration = (string) ANY" 
          */
-        /* All optional parameters
-         *
-         * "configuration-uri =" 
-         */
     )
     );
 
@@ -88,12 +76,15 @@ gst_rtp_vorbis_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_vorbis_depay_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_vorbis_depay_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_vorbis_depay_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_vorbis_depay_src_template);
 
-  gst_element_class_set_details (element_class, &gst_rtp_vorbis_depay_details);
+  gst_element_class_set_details_simple (element_class, "RTP Vorbis depayloader",
+      "Codec/Depayloader/Network/RTP",
+      "Extracts Vorbis Audio from RTP packets (RFC 5215)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -124,6 +115,33 @@ gst_rtp_vorbis_depay_init (GstRtpVorbisDepay * rtpvorbisdepay,
 {
   rtpvorbisdepay->adapter = gst_adapter_new ();
 }
+
+static void
+free_config (GstRtpVorbisConfig * conf)
+{
+  GList *headers;
+
+  for (headers = conf->headers; headers; headers = g_list_next (headers)) {
+    GstBuffer *header = GST_BUFFER_CAST (headers->data);
+
+    gst_buffer_unref (header);
+  }
+  g_list_free (conf->headers);
+  g_free (conf);
+}
+
+static void
+free_indents (GstRtpVorbisDepay * rtpvorbisdepay)
+{
+  GList *walk;
+
+  for (walk = rtpvorbisdepay->configs; walk; walk = g_list_next (walk)) {
+    free_config ((GstRtpVorbisConfig *) walk->data);
+  }
+  g_list_free (rtpvorbisdepay->configs);
+  rtpvorbisdepay->configs = NULL;
+}
+
 static void
 gst_rtp_vorbis_depay_finalize (GObject * object)
 {
@@ -134,70 +152,20 @@ gst_rtp_vorbis_depay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static const guint8 a2bin[256] = {
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 62, 64, 64, 64, 63,
-  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 64, 64, 64, 64, 64, 64,
-  64, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 64, 64, 64, 64, 64,
-  64, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64
-};
-
-static guint
-decode_base64 (const gchar * in, guint8 * out)
-{
-  guint8 v1, v2;
-  guint len = 0;
-
-  v1 = a2bin[(gint) * in];
-  while (v1 <= 63) {
-    /* read 4 bytes, write 3 bytes, invalid base64 are zeroes */
-    v2 = a2bin[(gint) * ++in];
-    *out++ = (v1 << 2) | ((v2 & 0x3f) >> 4);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v2 << 4) | ((v1 & 0x3f) >> 2);
-    v2 = (v1 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v1 << 6) | (v2 & 0x3f);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    len += 3;
-  }
-  /* move to '\0' */
-  while (*in != '\0')
-    in++;
-
-  /* subtract padding */
-  while (len > 0 && *--in == '=')
-    len--;
-
-  return len;
-}
-
+/* takes ownership of confbuf */
 static gboolean
 gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
-    const gchar * configuration)
+    GstBuffer * confbuf)
 {
   GstBuffer *buf;
   guint32 num_headers;
   guint8 *data;
   guint size;
+  guint offset;
   gint i, j;
 
-  /* deserialize base64 to buffer */
-  size = strlen (configuration);
-  GST_DEBUG_OBJECT (rtpvorbisdepay, "base64 config size %u", size);
-
-  data = g_malloc (size);
-  size = decode_base64 (configuration, data);
+  data = GST_BUFFER_DATA (confbuf);
+  size = GST_BUFFER_SIZE (confbuf);
 
   GST_DEBUG_OBJECT (rtpvorbisdepay, "config size %u", size);
 
@@ -220,6 +188,7 @@ gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
   num_headers = GST_READ_UINT32_BE (data);
   size -= 4;
   data += 4;
+  offset = 4;
 
   GST_DEBUG_OBJECT (rtpvorbisdepay, "have %u headers", num_headers);
 
@@ -253,6 +222,7 @@ gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
     guint8 n_headers, b;
     GstRtpVorbisConfig *conf;
     guint *h_sizes;
+    guint extra = 1;
 
     if (size < 6)
       goto too_small;
@@ -262,11 +232,15 @@ gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
     n_headers = data[5];
     size -= 6;
     data += 6;
+    offset += 6;
 
     GST_DEBUG_OBJECT (rtpvorbisdepay,
         "header %d, ident 0x%08x, length %u, left %u", i, ident, length, size);
 
-    if (size < length)
+    /* FIXME check if we already got this ident */
+
+    /* length might also include count of following size fields */
+    if (size < length && size + 1 != length)
       goto too_small;
 
     /* read header sizes we read 2 sizes, the third size (for which we allocate
@@ -280,10 +254,16 @@ gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
         if (size < 1)
           goto too_small;
         b = *data++;
+        offset++;
+        extra++;
         size--;
         h_size = (h_size << 7) | (b & 0x7f);
       } while (b & 0x80);
       GST_DEBUG_OBJECT (rtpvorbisdepay, "headers %d: size: %u", j, h_size);
+
+      if (length < h_size)
+        goto too_small;
+
       h_sizes[j] = h_size;
       length -= h_size;
     }
@@ -299,20 +279,28 @@ gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
       guint h_size;
 
       h_size = h_sizes[j];
-      if (size < h_size)
-        goto too_small;
+      if (size < h_size) {
+        if (j != n_headers || size + extra != h_size) {
+          free_config (conf);
+          goto too_small;
+        } else {
+          /* otherwise means that overall length field contained total length,
+           * including extra fields */
+          h_size -= extra;
+        }
+      }
 
       GST_DEBUG_OBJECT (rtpvorbisdepay, "reading header %d, size %u", j,
           h_size);
 
-      buf = gst_buffer_new_and_alloc (h_size);
-      memcpy (GST_BUFFER_DATA (buf), data, h_size);
+      buf = gst_buffer_create_sub (confbuf, offset, h_size);
       conf->headers = g_list_append (conf->headers, buf);
-      data += h_size;
+      offset += h_size;
       size -= h_size;
     }
     rtpvorbisdepay->configs = g_list_append (rtpvorbisdepay->configs, conf);
   }
+  gst_buffer_unref (confbuf);
 
   return TRUE;
 
@@ -320,19 +308,46 @@ gst_rtp_vorbis_depay_parse_configuration (GstRtpVorbisDepay * rtpvorbisdepay,
 too_small:
   {
     GST_DEBUG_OBJECT (rtpvorbisdepay, "configuration too small");
+    gst_buffer_unref (confbuf);
     return FALSE;
   }
 }
 
+static gboolean
+gst_rtp_vorbis_depay_parse_inband_configuration (GstRtpVorbisDepay *
+    rtpvorbisdepay, guint ident, guint8 * configuration, guint size,
+    guint length)
+{
+  GstBuffer *confbuf;
+  guint8 *conf;
+
+  if (G_UNLIKELY (size < 4))
+    return FALSE;
+
+  /* transform inline to out-of-band and parse that one */
+  confbuf = gst_buffer_new_and_alloc (size + 9);
+  conf = GST_BUFFER_DATA (confbuf);
+  /* 1 header */
+  GST_WRITE_UINT32_BE (conf, 1);
+  /* write Ident */
+  GST_WRITE_UINT24_BE (conf + 4, ident);
+  /* write sort-of-length */
+  GST_WRITE_UINT16_BE (conf + 7, length);
+  /* copy remainder */
+  memcpy (conf + 9, configuration, size);
+
+  return gst_rtp_vorbis_depay_parse_configuration (rtpvorbisdepay, confbuf);
+}
+
 static gboolean
 gst_rtp_vorbis_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstStructure *structure;
   GstRtpVorbisDepay *rtpvorbisdepay;
   GstCaps *srccaps;
-  const gchar *delivery_method;
   const gchar *configuration;
   gint clock_rate;
+  gboolean res;
 
   rtpvorbisdepay = GST_RTP_VORBIS_DEPAY (depayload);
 
@@ -342,56 +357,37 @@ gst_rtp_vorbis_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
     goto no_rate;
 
-  /* see how the configuration parameters will be transmitted */
-  delivery_method = gst_structure_get_string (structure, "delivery-method");
-  if (delivery_method == NULL)
-    goto no_delivery_method;
-
-  if (g_strcasecmp (delivery_method, "inline")) {
-    /* configure string is in the caps */
-  } else if (g_strcasecmp (delivery_method, "in_band")) {
-    /* headers will (also) be transmitted in the RTP packets */
-  } else if (g_str_has_prefix (delivery_method, "out_band/")) {
-    /* some other method of header delivery. */
-    goto unsupported_delivery_method;
-  } else
-    goto unsupported_delivery_method;
-
   /* read and parse configuration string */
   configuration = gst_structure_get_string (structure, "configuration");
-  if (configuration == NULL)
-    goto no_configuration;
+  if (configuration) {
+    GstBuffer *confbuf;
+    guint8 *data;
+    gsize size;
 
-  if (!gst_rtp_vorbis_depay_parse_configuration (rtpvorbisdepay, configuration))
-    goto invalid_configuration;
+    /* deserialize base64 to buffer */
+    data = g_base64_decode (configuration, &size);
+
+    confbuf = gst_buffer_new ();
+    GST_BUFFER_DATA (confbuf) = data;
+    GST_BUFFER_MALLOCDATA (confbuf) = data;
+    GST_BUFFER_SIZE (confbuf) = size;
+    if (!gst_rtp_vorbis_depay_parse_configuration (rtpvorbisdepay, confbuf))
+      goto invalid_configuration;
+  } else {
+    GST_WARNING_OBJECT (rtpvorbisdepay, "no configuration specified");
+  }
 
   /* caps seem good, configure element */
   depayload->clock_rate = clock_rate;
 
   /* set caps on pad and on header */
   srccaps = gst_caps_new_simple ("audio/x-vorbis", NULL);
-  gst_pad_set_caps (depayload->srcpad, srccaps);
+  res = gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
-  return TRUE;
+  return res;
 
   /* ERRORS */
-unsupported_delivery_method:
-  {
-    GST_ERROR_OBJECT (rtpvorbisdepay,
-        "unsupported delivery-method \"%s\" specified", delivery_method);
-    return FALSE;
-  }
-no_delivery_method:
-  {
-    GST_ERROR_OBJECT (rtpvorbisdepay, "no delivery-method specified");
-    return FALSE;
-  }
-no_configuration:
-  {
-    GST_ERROR_OBJECT (rtpvorbisdepay, "no configuration specified");
-    return FALSE;
-  }
 invalid_configuration:
   {
     GST_ERROR_OBJECT (rtpvorbisdepay, "invalid configuration specified");
@@ -451,13 +447,9 @@ gst_rtp_vorbis_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   guint32 timestamp;
   guint32 header, ident;
   guint8 F, VDT, packets;
-  gboolean free_payload;
 
   rtpvorbisdepay = GST_RTP_VORBIS_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   payload_len = gst_rtp_buffer_get_payload_len (buf);
 
   GST_DEBUG_OBJECT (depayload, "got RTP packet of size %d", payload_len);
@@ -467,7 +459,6 @@ gst_rtp_vorbis_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     goto packet_short;
 
   payload = gst_rtp_buffer_get_payload (buf);
-  free_payload = FALSE;
 
   header = GST_READ_UINT32_BE (payload);
   /*
@@ -590,6 +581,15 @@ gst_rtp_vorbis_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     if (G_UNLIKELY (length > payload_len))
       goto length_short;
 
+    /* handle in-band configuration */
+    if (G_UNLIKELY (VDT == 1)) {
+      GST_DEBUG_OBJECT (rtpvorbisdepay, "in-band configuration");
+      if (!gst_rtp_vorbis_depay_parse_inband_configuration (rtpvorbisdepay,
+              ident, payload, payload_len, length))
+        goto invalid_configuration;
+      goto no_output;
+    }
+
     /* create buffer for packet */
     if (G_UNLIKELY (to_free)) {
       outbuf = gst_buffer_new ();
@@ -628,15 +628,9 @@ no_output:
     return NULL;
   }
   /* ERORRS */
-bad_packet:
-  {
-    GST_ELEMENT_WARNING (rtpvorbisdepay, STREAM, DECODE,
-        (NULL), ("Packet did not validate"));
-    return NULL;
-  }
 switch_failed:
   {
-    GST_ELEMENT_ERROR (rtpvorbisdepay, STREAM, DECODE,
+    GST_ELEMENT_WARNING (rtpvorbisdepay, STREAM, DECODE,
         (NULL), ("Could not switch codebooks"));
     return NULL;
   }
@@ -657,6 +651,13 @@ length_short:
         (NULL), ("Packet contains invalid data"));
     return NULL;
   }
+invalid_configuration:
+  {
+    /* fatal, as we otherwise risk carrying on without output */
+    GST_ELEMENT_ERROR (rtpvorbisdepay, STREAM, DECODE,
+        (NULL), ("Packet contains invalid configuration"));
+    return NULL;
+  }
 }
 
 static GstStateChangeReturn
@@ -680,6 +681,9 @@ gst_rtp_vorbis_depay_change_state (GstElement * element,
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      free_indents (rtpvorbisdepay);
+      break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
     default:
@@ -692,5 +696,5 @@ gboolean
 gst_rtp_vorbis_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpvorbisdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_VORBIS_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_VORBIS_DEPAY);
 }
