@@ -44,11 +44,7 @@
 /**
  * SECTION:element-rtpdec
  *
- * <refsect2>
- * <para>
  * A simple RTP session manager used internally by rtspsrc.
- * </para>
- * </refsect2>
  *
  * Last reviewed on 2006-06-20 (0.10.4)
  */
@@ -62,17 +58,11 @@
 #endif
 
 #include "gstrtpdec.h"
+#include <stdio.h>
 
 GST_DEBUG_CATEGORY_STATIC (rtpdec_debug);
 #define GST_CAT_DEFAULT (rtpdec_debug)
 
-/* elementfactory information */
-static const GstElementDetails rtpdec_details =
-GST_ELEMENT_DETAILS ("RTP Decoder",
-    "Codec/Parser/Network",
-    "Accepts raw RTP and RTCP packets and sends them forward",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* GstRTPDec signals and args */
 enum
 {
@@ -211,23 +201,26 @@ gst_rtp_dec_base_init (gpointer klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   /* sink pads */
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_dec_recv_rtp_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_dec_recv_rtcp_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_dec_recv_rtp_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_dec_recv_rtcp_sink_template);
   /* src pads */
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_dec_recv_rtp_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_dec_rtcp_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_dec_recv_rtp_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_dec_rtcp_src_template);
 
-  gst_element_class_set_details (element_class, &rtpdec_details);
+  gst_element_class_set_details_simple (element_class, "RTP Decoder",
+      "Codec/Parser/Network",
+      "Accepts raw RTP and RTCP packets and sends them forward",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 /* BOXED:UINT,UINT */
 #define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
 
-void
+static void
 gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
     GValue * return_value,
     guint n_param_values,
@@ -252,8 +245,8 @@ gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
     data2 = closure->data;
   }
   callback =
-      (GMarshalFunc_BOXED__UINT_UINT) (marshal_data ? marshal_data : cc->
-      callback);
+      (GMarshalFunc_BOXED__UINT_UINT) (marshal_data ? marshal_data :
+      cc->callback);
 
   v_return = callback (data1,
       g_marshal_value_peek_uint (param_values + 1),
@@ -262,7 +255,7 @@ gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
   g_value_take_boxed (return_value, v_return);
 }
 
-void
+static void
 gst_rtp_dec_marshal_VOID__UINT_UINT (GClosure * closure,
     GValue * return_value,
     guint n_param_values,
@@ -285,8 +278,8 @@ gst_rtp_dec_marshal_VOID__UINT_UINT (GClosure * closure,
     data2 = closure->data;
   }
   callback =
-      (GMarshalFunc_VOID__UINT_UINT) (marshal_data ? marshal_data : cc->
-      callback);
+      (GMarshalFunc_VOID__UINT_UINT) (marshal_data ? marshal_data :
+      cc->callback);
 
   callback (data1,
       g_marshal_value_peek_uint (param_values + 1),
@@ -311,7 +304,7 @@ gst_rtp_dec_class_init (GstRTPDecClass * g_class)
   g_object_class_install_property (gobject_class, PROP_LATENCY,
       g_param_spec_uint ("latency", "Buffer latency in ms",
           "Amount of ms to buffer", 0, G_MAXUINT, DEFAULT_LATENCY_MS,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstRTPDec::request-pt-map:
@@ -446,11 +439,8 @@ gst_rtp_dec_finalize (GObject * object)
 static gboolean
 gst_rtp_dec_query_src (GstPad * pad, GstQuery * query)
 {
-  GstRTPDec *rtpdec;
   gboolean res;
 
-  rtpdec = GST_RTP_DEC (GST_PAD_PARENT (pad));
-
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_LATENCY:
     {
@@ -548,6 +538,7 @@ bad_packet:
   {
     GST_ELEMENT_WARNING (rtpdec, STREAM, DECODE, (NULL),
         ("RTP packet did not validate, dropping"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 }
@@ -702,9 +693,11 @@ gst_rtp_dec_chain_rtcp (GstPad * pad, GstBuffer * buffer)
 bad_packet:
   {
     GST_WARNING_OBJECT (src, "got invalid RTCP packet");
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 #else
+  gst_buffer_unref (buffer);
   return GST_FLOW_OK;
 #endif
 }
@@ -759,9 +752,6 @@ static GstStateChangeReturn
 gst_rtp_dec_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
-  GstRTPDec *rtpdec;
-
-  rtpdec = GST_RTP_DEC (element);
 
   switch (transition) {
     default:
