@@ -79,9 +79,15 @@ typedef struct _GstRTSPSrcClass GstRTSPSrcClass;
 #define GST_RTSP_STREAM_LOCK(rtsp)       (g_static_rec_mutex_lock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
 #define GST_RTSP_STREAM_UNLOCK(rtsp)     (g_static_rec_mutex_unlock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
 
-#define GST_RTSP_CONN_GET_LOCK(rtsp)     (GST_RTSPSRC_CAST(rtsp)->conn_rec_lock)
-#define GST_RTSP_CONN_LOCK(rtsp)         (g_static_rec_mutex_lock (GST_RTSP_CONN_GET_LOCK(rtsp)))
-#define GST_RTSP_CONN_UNLOCK(rtsp)       (g_static_rec_mutex_unlock (GST_RTSP_CONN_GET_LOCK(rtsp)))
+typedef struct _GstRTSPConnInfo GstRTSPConnInfo;
+
+struct _GstRTSPConnInfo {
+  gchar              *location;
+  GstRTSPUrl         *url;
+  gchar              *url_str;
+  GstRTSPConnection  *connection;
+  gboolean            connected;
+};
 
 typedef struct _GstRTSPStream GstRTSPStream;
 
@@ -106,6 +112,7 @@ struct _GstRTSPStream {
   /* our udp sources */
   GstElement   *udpsrc[2];
   GstPad       *blockedpad;
+  gboolean      is_ipv6;
 
   /* our udp sinks back to the server */
   GstElement   *udpsink[2];
@@ -116,19 +123,29 @@ struct _GstRTSPStream {
 
   /* state */
   gint          pt;
+  guint         port;
   gboolean      container;
   /* original control url */
   gchar        *control_url;
-  /* fully qualified control url */
-  gchar        *setup_url;
-  guint32       ssrc; 
+  guint32       ssrc;
   guint32       seqbase;
   guint64       timebase;
 
+  /* per stream connection */
+  GstRTSPConnInfo  conninfo;
+
+  /* session */
+  GObject      *session;
+
   /* bandwidth */
   guint         as_bandwidth;
   guint         rs_bandwidth;
   guint         rr_bandwidth;
+
+  /* destination */
+  gchar        *destination;
+  gboolean      is_multicast;
+  guint         ttl;
 };
 
 /**
@@ -163,25 +180,23 @@ struct _GstRTSPSrc {
   /* UDP mode loop */
   gint             loop_cmd;
   gboolean         ignore_timeout;
+  gboolean         waiting;
+  gboolean         open_error;
 
   /* mutex for protecting state changes */
   GStaticRecMutex *state_rec_lock;
 
-  /* mutex for protecting the connection */
-  GStaticRecMutex *conn_rec_lock;
-
+  GstSDPMessage   *sdp;
+  gboolean         from_sdp;
   gint             numstreams;
   GList           *streams;
   GstStructure    *props;
   gboolean         need_activate;
 
   /* properties */
-  gchar            *location;
-  gchar            *req_location; /* Sanitised URL to use in network requests */
-  GstRTSPUrl       *url;
   GstRTSPLowerTrans protocols;
   gboolean          debug;
-  guint   	    retry;
+  guint             retry;
   guint64           udp_timeout;
   GTimeVal          tcp_timeout;
   GTimeVal         *ptcp_timeout;
@@ -196,6 +211,10 @@ struct _GstRTSPSrc {
   guint             rtp_blocksize;
   gchar            *user_id;
   gchar            *user_pw;
+  gint              buffer_mode;
+  GstRTSPRange      client_port_range;
+  gint              udp_buffer_size;
+  gboolean          short_header;
 
   /* state */
   GstRTSPState       state;
@@ -205,17 +224,21 @@ struct _GstRTSPSrc {
   gchar             *addr;
   gboolean           need_redirect;
   GstRTSPTimeRange  *range;
+  gchar             *control;
+  guint              next_port_num;
 
   /* supported methods */
   gint               methods;
 
-  /* session management */
-  GstElement      *session;
-  gulong           session_sig_id;
-  gulong           session_ptmap_id;
+  gboolean           seekable;
+  GstClockTime       last_pos;
 
-  GstRTSPConnection  *connection;
-  gboolean            connected;
+  /* session management */
+  GstElement      *manager;
+  gulong           manager_sig_id;
+  gulong           manager_ptmap_id;
+
+  GstRTSPConnInfo  conninfo;
 
   /* a list of RTSP extensions as GstElement */
   GstRTSPExtensionList  *extensions;
