@@ -85,7 +85,7 @@
 - (void) sendEvent:(NSEvent *) event {
   BOOL taken = NO;
 
-  GST_DEBUG ("event %p type:%d", event,[event type]);
+  GST_DEBUG ("event %p type:%d", event,(gint)[event type]);
 
   if ([event type] == NSKeyDown) {
   }
@@ -128,6 +128,7 @@
   }
 
   self = [super initWithFrame: frame pixelFormat:fmt];
+  [fmt release];
 
    actualContext = [self openGLContext];
    [actualContext makeCurrentContext];
@@ -383,6 +384,37 @@
   [self initTextures];
 }
 
+- (void) haveSuperviewReal:(NSMutableArray *)closure {
+	BOOL haveSuperview = [self superview] != nil;
+	[closure addObject:[NSNumber numberWithBool:haveSuperview]];
+}
+
+- (BOOL) haveSuperview {
+	NSMutableArray *closure = [NSMutableArray arrayWithCapacity:1];
+	[self performSelectorOnMainThread:@selector(haveSuperviewReal:)
+			withObject:(id)closure waitUntilDone:YES];
+
+	return [[closure objectAtIndex:0] boolValue];
+}
+
+- (void) addToSuperviewReal:(NSView *)superview {
+	NSRect bounds;
+	[superview addSubview:self];
+	bounds = [superview bounds];
+	[self setFrame:bounds];
+	[self setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
+}
+
+- (void) addToSuperview: (NSView *)superview {
+	[self performSelectorOnMainThread:@selector(addToSuperviewReal:)
+			withObject:superview waitUntilDone:YES];
+}
+
+- (void) removeFromSuperview: (id)unused
+{
+	[self removeFromSuperview];
+}
+
 - (void) dealloc {
   GST_LOG ("dealloc called");
   if (data) g_free(data);
