@@ -12,7 +12,7 @@ static guint spect_bands = 256;
 static gfloat height_scale = 2.0;
 
 static void
-on_window_destroy (GtkObject * object, gpointer user_data)
+on_window_destroy (GObject * object, gpointer user_data)
 {
   drawingarea = NULL;
   gtk_main_quit ();
@@ -79,22 +79,29 @@ draw_spectrum (gfloat * data)
 {
   gint i;
   GdkRectangle rect = { 0, 0, spect_bands, spect_height };
+  cairo_t *cr;
 
   if (!drawingarea)
     return;
 
-  gdk_window_begin_paint_rect (drawingarea->window, &rect);
-  gdk_draw_rectangle (drawingarea->window, drawingarea->style->black_gc,
-      TRUE, 0, 0, spect_bands, spect_height);
+  gdk_window_begin_paint_rect (gtk_widget_get_window (drawingarea), &rect);
+  cr = gdk_cairo_create (gtk_widget_get_window (drawingarea));
+
+  cairo_set_source_rgb (cr, 0, 0, 0);
+  cairo_rectangle (cr, 0, 0, spect_bands, spect_height);
+  cairo_fill (cr);
   for (i = 0; i < spect_bands; i++) {
-    gdk_draw_rectangle (drawingarea->window, drawingarea->style->white_gc,
-        TRUE, i, -data[i], 1, spect_height + data[i]);
+    cairo_set_source_rgb (cr, 1, 1, 1);
+    cairo_rectangle (cr, i, -data[i], 1, spect_height + data[i]);
+    cairo_fill (cr);
   }
-  gdk_window_end_paint (drawingarea->window);
+  cairo_destroy (cr);
+
+  gdk_window_end_paint (gtk_widget_get_window (drawingarea));
 }
 
 /* receive spectral data from element message */
-gboolean
+static gboolean
 message_handler (GstBus * bus, GstMessage * message, gpointer data)
 {
   if (message->type == GST_MESSAGE_ELEMENT) {
@@ -102,7 +109,7 @@ message_handler (GstBus * bus, GstMessage * message, gpointer data)
     const gchar *name = gst_structure_get_name (s);
 
     if (strcmp (name, "spectrum") == 0) {
-      gfloat spect[spect_bands];
+      gfloat *spect = g_new (gfloat, spect_bands);
       const GValue *list;
       const GValue *value;
       guint i;
@@ -113,6 +120,7 @@ message_handler (GstBus * bus, GstMessage * message, gpointer data)
         spect[i] = height_scale * g_value_get_float (value);
       }
       draw_spectrum (spect);
+      g_free (spect);
     }
   }
   return TRUE;
